package infomat;

import infomat.algorithms.grouping.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.weighting.*;

import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;



/**
 * Simple class for running clustering without the GUI.
 *
 * <br>
 * <br>
 * <code>java -cp classes/ infomat.ExampleClusterer</code>
 *
 * @version          081001
 * @author           Magnus Rosell
 *
 */
public class ExampleClusterer{

    public static void main(String[] args) throws IOException, PropertyNotFoundException{

	// Load the matrix
	// This one has texts as rows and words as columns
	String matrixFileName = "examples/data/eng_matrix.xml";
	File matrixFile = new File(matrixFileName);
	IMatrixIO iomatrix = new IMatrixIO();
	IMatrix matrix = iomatrix.load(matrixFile);

	// Load a categorization
	String categorizationFileName = "examples/data/eng_text_grouping.xml";
	File categorizationFile = new File(categorizationFileName);
	IObjectGroupingIO iobio = new IObjectGroupingIO();
	boolean rows=true; // row categorization
	IObjectGrouping categorization = 
	    iobio.load(rows, matrix, categorizationFile);
	

	// Filter the matrix
	// The example is to small
	/*
	IMatrixCellFilter matrixCellFilter = new IMatrixCellFilter();
	matrixCellFilter.filter(matrix);
	*/

	// Weight the matrix
	TFIDFWeighter weighter = new TFIDFWeighter();
	weighter.weight(matrix);

	// Cluster the rows (texts) of the matrix
	IObjectSet rowSet = matrix.getRowIObjectSet();

	// Similarity
	boolean row = true; // Similarity between the rows of the matrix.
	SparseDotSimilarity dotSim = new SparseDotSimilarity(matrix, row);

	// Cluster
	KMeans kmeans = new KMeans(dotSim);
	IObjectGrouping clustering = kmeans.cluster(rowSet);
	clustering.removeEmptyGroups();

	/*
	try{
	    kmeans.setProperty("Number of clusters", 2);
	}catch(PropertyNotFoundException pnfe){
	    // Do nothing
	    System.out.println("PropertyNotFoundException!");
	}
	TopDownSplitter tds = new TopDownSplitter();
	tds.setSimilarity(dotSim);
	IObjectGrouping clustering = tds.cluster(rowSet, kmeans);
	clustering.removeEmptyGroups();
	*/

	/*
	BisectingKMeans bkm = new BisectingKMeans(dotSim);
	IObjectGrouping clustering = bkm.cluster(rowSet);
	clustering.removeEmptyGroups();
	*/


	// The current time
	long time = System.currentTimeMillis();
	
	// Write Text Result
	String textResultFileName = 
	    "results/clusteringResult."+time+".xml";
	File textResFile = new File(textResultFileName);
	IObjectGroupingTextResult gtio = new IObjectGroupingTextResult();
	gtio.save(matrix, clustering, dotSim, textResFile);

	// Write Full Clustering Result
	String resultFileName = 
	    "results/clustering."+time+".xml";
	File resFile = new File(resultFileName);
	IObjectGroupingIO iogingio = new IObjectGroupingIO();
	iogingio.save(clustering, matrix, resFile);

	// Evaluate and write the results
	String evaluationFileName = 
	    "results/clusteringEvaluation."+time+".xml";
	Evaluation eval = 
	    new Evaluation(matrix, clustering, categorization, dotSim, ""+time);
	calculateAndSaveEvaluation(eval, evaluationFileName);
	
	System.out.println("Clustered!!!");

    }


    private static void calculateAndSaveEvaluation(Evaluation eval, 
						   String resFileName){
	Measures measures = new Measures("Measures");
	try{
	    measures = eval.calculateAllMeasures();
	}
	catch(MeasureNameConflictException mnce){
	    System.out.println("Measure Name Conflict!");
	}
	measures.save(resFileName);
    }



}
