package infomat.algorithms.grouping;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import mro.util.*;
import mro.util.properties.*;

import java.util.*;
import java.io.*; 

/*
  import java.util.Vector; 
  import java.util.Map.Entry; 
  import java.util.HashMap; 
  import java.util.HashSet; 
  import java.util.Set; 
*/

/**
 * Groups IObjects depeneding on their location-string. 
 *      
 * @version          080125
 * @author           Magnus Rosell
 */

public class LocationGrouper extends GroupingAlgorithm{ 

    /*
      protected Properties properties;
      protected UniqueIdentifier identifier;
    */

    private static int locationGroupingNumber = 1;
	
    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = 
	    new mro.util.properties.Properties("Location Grouping Properties", CurrentClassGetter.getClassName());
	try{

	    Property p = new Property("Location string divider", "path-separator", "How the string is divided");
	    PropertyValueDescription pvd = new PropertyValueDescription("path-separator", "Considering the location string a file path");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    /*
	    p = new Property("Length of group name", "short", "Length of generated group name");
	    pvd = new PropertyValueDescription("short", "Shortest unique");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("long", "The whole length.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);
	    */
			
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }

    public LocationGrouper(){
	this.properties = getDefaultProperties();
    }
	
    public LocationGrouper(mro.util.properties.Properties properties){
	this.properties=properties;
    }
	
    /*
    private String locationDivider = "\\\\/"; //File.separator; //"/";

    private String longestCommonStart(String s1, String s2){
	String ret = "";
	String[] s1s = s1.split("["+locationDivider+"]");
	String[] s2s = s2.split("["+locationDivider+"]");
	for(int i=0; i<s1s.length && i<s2s.length; i++){
	    if(s1s[i].equals(s2s[i]))
		ret += s1s[i]+File.separator; //locationDivider;
	    else
		break;
	}
	return ret;
    }

    private String removeLastPart(String s){
	String[] ll = s.split("["+locationDivider+"]");
	String ret = "";
	for(int i=0; i<ll.length-1; i++){
	    ret += ll[i]+File.separator; //locationDivider;
	}
	return ret;
    }
    */

	
    public IObjectGrouping cluster(IObjectGroup iogroup){

	HashMap<IObject, String> objectGroupNameMap = 
	    new HashMap<IObject, String>();
	Vector<IObject> ios = iogroup.getOrdered();
	IObject io = ios.elementAt(0);
	String longestCommon = io.getLocation();
	if(longestCommon==null)
	    return null;
	longestCommon = PathHelper.removeLastPart(longestCommon);
	objectGroupNameMap.put(io, longestCommon);
	for(int i=1; i<ios.size(); i++){
	    io = ios.elementAt(i);
	    String l = io.getLocation();
	    l = PathHelper.removeLastPart(l);
	    if(l==null)
		return null;
	    longestCommon = PathHelper.longestCommonStart(longestCommon, l);
	    objectGroupNameMap.put(io, l);
	}
	
	Vector<IObjectGroup> groups = new Vector<IObjectGroup>();
	HashMap<String, IObjectGroup> nameGroupMap = 
	    new HashMap<String, IObjectGroup>();
	for(int i=0; i<ios.size(); i++){
	    io = ios.elementAt(i);
	    String l = objectGroupNameMap.get(io);
	    l = l.substring(longestCommon.length(), l.length());
	    //	    objectGroupNameMap.put(io, l);
	    IObjectGroup iog = nameGroupMap.get(l);
	    if(iog==null){
		iog = new IObjectGroup(l);
		nameGroupMap.put(l, iog);
		groups.add(iog);
	    }
	    iog.add(io);
	}	

	String name = "Location Grouping "+locationGroupingNumber;
	locationGroupingNumber++;
	IObjectGrouping grouping = new IObjectGrouping(name);

	for(int i=0; i<groups.size(); i++){
	    IObjectGroup iog = groups.elementAt(i);
	    grouping.add(iog);
	}
	
	return grouping;
    }


    /*
    public IObjectGrouping cluster(IObjectSet ioset){

	IObjectGrouping grouping = cluster((IObjectGroup) ioset);
	String name = grouping.getName();
	int i=1;
	while(!ioset.setGrouping(grouping)){
	    grouping.setName(name+" ("+i+")");
	    i++;
	}
	return grouping;
    }
    */
	
    /*
      private void printGroup(IObjectGroup group){
      System.out.println(group.getString());
      Vector ordered = group.getOrdered();
      for(int i=0; i<ordered.size(); i++)
      System.out.println(" * "+((IObject) ordered.elementAt(i)).getString());
		
      }
	
	
      private float compareObjectToCentroid(HashMap<IObject, Float> representation,
      HashMap<IObject, Float> centroid){
      float sim =0f;
      Set<Map.Entry<IObject, Float>> set = representation.entrySet();
      Iterator<Map.Entry<IObject, Float>> it = set.iterator();
      while(it.hasNext()){
      Map.Entry<IObject, Float> entry = it.next();
      Float f = centroid.get(entry.getKey());
      if(f!=null){
      sim+= f * entry.getValue();
      }
      }
      return sim;
      }
    */


	
    public String getInfoString(){
    	return "Location Grouper";
    }

    public String getAlgorithmName(){
    	return "Location Grouper";
    }
    
    public Vector getPropertyVector(){
    	return properties.getPropertyVector();
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }

    public UniqueIdentifier getUniqueIdentifier(){
    	return identifier;
    }

    public void setUniqueIdentifier(UniqueIdentifier uniqueIdentifier){
    	this.identifier=uniqueIdentifier;
    }

}













/*
public class LocationGrouper extends GroupingAlgorithm{ 


    private static int locationGroupingNumber = 1;
	
    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = 
	    new mro.util.properties.Properties("Location Grouping Properties", CurrentClassGetter.getClassName());
	try{

	    Property p = new Property("Location string divider", "path-separator", "How the string is divided");
	    PropertyValueDescription pvd = new PropertyValueDescription("path-separator", "Considering the location string a file path");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }

    public LocationGrouper(){
	this.properties = getDefaultProperties();
    }
	
    public LocationGrouper(mro.util.properties.Properties properties){
	this.properties=properties;
    }
	

    private String locationDivider = File.separator; //"/";

    private String longestCommonStart(String s1, String s2){
	String ret = "";
	String[] s1s = s1.split(locationDivider);
	String[] s2s = s2.split(locationDivider);
	for(int i=0; i<s1s.length && i<s2s.length; i++){
	    if(s1s[i].equals(s2s[i]))
		ret += s1s[i]+locationDivider;
	    else
		break;
	}
	return ret;
    }

    private String removeLastPart(String s){
	String[] ll = s.split(locationDivider);
	String ret = "";
	for(int i=0; i<ll.length-1; i++){
	    ret += ll[i]+locationDivider;
	}
	return ret;
    }


	
    public IObjectGrouping cluster(IObjectGroup iogroup){

	HashMap<IObject, String> objectGroupNameMap = 
	    new HashMap<IObject, String>();
	Vector<IObject> ios = iogroup.getOrdered();
	IObject io = ios.elementAt(0);
	String longestCommon = io.getLocation();
	longestCommon = removeLastPart(longestCommon);
	objectGroupNameMap.put(io, longestCommon);
	for(int i=1; i<ios.size(); i++){
	    io = ios.elementAt(i);
	    String l = io.getLocation();
	    l = removeLastPart(l);
	    longestCommon = longestCommonStart(longestCommon, l);
	    objectGroupNameMap.put(io, l);
	}
	
	Vector<IObjectGroup> groups = new Vector<IObjectGroup>();
	HashMap<String, IObjectGroup> nameGroupMap = 
	    new HashMap<String, IObjectGroup>();
	for(int i=0; i<ios.size(); i++){
	    io = ios.elementAt(i);
	    String l = objectGroupNameMap.get(io);
	    l = l.substring(longestCommon.length(), l.length());
	    //	    objectGroupNameMap.put(io, l);
	    IObjectGroup iog = nameGroupMap.get(l);
	    if(iog==null){
		iog = new IObjectGroup(l);
		nameGroupMap.put(l, iog);
		groups.add(iog);
	    }
	    iog.add(io);
	}	

	String name = "Location Grouping "+locationGroupingNumber;
	locationGroupingNumber++;
	IObjectGrouping grouping = new IObjectGrouping(name);

	for(int i=0; i<groups.size(); i++){
	    IObjectGroup iog = groups.elementAt(i);
	    grouping.add(iog);
	}
	
	return grouping;
    }


    public IObjectGrouping cluster(IObjectSet ioset){

	IObjectGrouping grouping = cluster((IObjectGroup) ioset);
	String name = grouping.getName();
	int i=1;
	while(!ioset.setGrouping(grouping)){
	    grouping.setName(name+" ("+i+")");
	    i++;
	}
	return grouping;
    }


	
    public String getInfoString(){
    	return "Location Grouper";
    }

    public String getAlgorithmName(){
    	return "Location Grouper";
    }
    
    public Vector getPropertyVector(){
    	return properties.getPropertyVector();
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }

    public UniqueIdentifier getUniqueIdentifier(){
    	return identifier;
    }

    public void setUniqueIdentifier(UniqueIdentifier uniqueIdentifier){
    	this.identifier=uniqueIdentifier;
    }

}
*/
