package infomat.algorithms.projection;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.ivector.*;

import mro.util.properties.*;

import java.util.HashMap;
import java.util.Vector;

/**
 * A class for projecting objects onto a grouping, 
 * using a similarity measure.
 *
 * @version          081217
 * @author           Magnus Rosell
 */
public class Projector{

    public static ProjectionSimilarity project(IObjectGroup group, IObjectGrouping grouping, ISimilarity similarity){
	if(similarity instanceof SparseISimilarity)
	    return projectSparse(group, grouping, (SparseISimilarity) similarity);	
	if(similarity instanceof DenseISimilarity)
	    return projectDense(group, grouping, (DenseISimilarity) similarity);
	return null;
    }


    public static ProjectionSimilarity projectSparse(IObjectGroup group, IObjectGrouping grouping, SparseISimilarity similarity){

	ProjectionSimilarity projSim = new ProjectionSimilarity();

	Vector<IObjectGroup> groups = grouping.getGroups();
	Vector<IVector<IVectorIndex>> centroids = new Vector<IVector<IVectorIndex>>();
	for(int j=0; j<groups.size(); j++){
	    IObjectGroup iog = groups.get(j);
	    IVector<IVectorIndex> centroid = 
		similarity.getGroupRepresentation(iog);
	    centroids.add(centroid);
	}

	Vector<IObject> ios = group.getOrdered();
	for(int i=0; i<ios.size(); i++){
	    IObject io = ios.get(i);
	    IVector<IVectorIndex> iorepr = similarity.getRepresentation(io);
	    float[] newiorepr = new float[groups.size()];
	    for(int j=0; j<groups.size(); j++){
		IVector<IVectorIndex> centroid = centroids.get(j);
		newiorepr[j] = similarity.sim(iorepr, centroid);
	    }
	    projSim.put(io, newiorepr);
	}

	return projSim;
    }


    public static ProjectionSimilarity projectDense(IObjectGroup group, IObjectGrouping grouping, DenseISimilarity similarity){

	ProjectionSimilarity projSim = new ProjectionSimilarity();

	Vector<IObjectGroup> groups = grouping.getGroups();
	Vector<float[]> centroids = new Vector<float[]>();
	for(int j=0; j<groups.size(); j++){
	    IObjectGroup iog = groups.get(j);
	    float[] centroid = 
		similarity.getGroupRepresentation(iog);
	    centroids.add(centroid);
	}

	Vector<IObject> ios = group.getOrdered();
	for(int i=0; i<ios.size(); i++){
	    IObject io = ios.get(i);
	    float[] iorepr = similarity.getRepresentation(io);
	    float[] newiorepr = new float[groups.size()];
	    for(int j=0; j<groups.size(); j++){
		float[] centroid = centroids.get(j);
		newiorepr[j] = similarity.sim(iorepr, centroid);
	    }
	    projSim.put(io, newiorepr);
	}

	return projSim;
    }


}
