package infomat.algorithms.reduction;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.ivector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;


/**
 * Reduces the IObjectGroup:s of a IObjectGrouping relative
 * the IObjectGroup:s of another IObjectGrouping.
 **/
public class Reducer implements HasProperties{

    private static int reductionNumber = 1;
    private mro.util.properties.Properties properties;

    public Reducer(){
	this.properties = getDefaultProperties();
    }

    public Reducer(mro.util.properties.Properties properties){
	this.properties = properties;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("Reducer Properties", CurrentClassGetter.getClassName());
		
	try{

	    Property p = new Property("Reduction Method", 0, "The reduction method");

	    PropertyValueDescription pvd = new PropertyValueDescription("0", "Each group reduced to \"Number objects\" in the current order.");
	    p.addValueDescription(pvd);

	    pvd = new PropertyValueDescription("1", "Similarity to the centroid. See \"Bigger similarity to the centorid than\".");
	    p.addValueDescription(pvd);

	    pvd = new PropertyValueDescription("2", "\"0\" and \"1\"");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    p = new Property("Number of objects", 10, "The maximum number of objects that will be saved for each group");
	    properties.addProperty(p);

	    p = new Property("Bigger similarity to centroid than", 0.0, "IObjects with bigger similarity to the centroid of each group are saved.");
	    properties.addProperty(p);


	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }


    /**
     * Note: does not handle missing zero-sized groups!!!
     */
    public IObjectGrouping reduceGrouping(SparseISimilarity similarity, IObjectGrouping ioging){
	
	if(ioging==null)
	    return null;

	Vector<IObjectGroup> groups = ioging.getGroups();

	if(groups.size()==0)
	    return null;

	Vector<IObjectGroup> newGroups = new Vector<IObjectGroup>();
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup newGroup = 
		reduceGroup(similarity, groups.elementAt(i));
	    newGroups.add(newGroup);
	}

	String name = "Reduced ("+ioging.getName()+") "+reductionNumber;
	reductionNumber++;
	IObjectGrouping newIOGING = 
	    new IObjectGrouping(name);

	newIOGING.addAllGroups(newGroups);
	return newIOGING;
    }
    
    /**
     * Makes a new IObjectGroup that consists of 
     * objects that have the specified relation to the relaive group.
     */
    public IObjectGroup reduceGroup(SparseISimilarity similarity, IObjectGroup iog){

	int reductionMethod = (Integer) getPropertyValue("Reduction Method");
	int maxNumOfObjects = (Integer) 
	    getPropertyValue("Number of objects");
	double minSim = (Double) getPropertyValue("Bigger similarity to centroid than");

	IObjectGroup newIOG = 
	    new IObjectGroup("Reduced ("+iog.getName()+")");

	IVector<IVectorIndex> groupRepr = 
	    similarity.getGroupRepresentation(iog);
	Vector<IObject> newIOS = new Vector<IObject>();
	Vector<IObject> ordered = iog.getOrdered();
	HashMap<IObject, Float> objectOrderValueMap = 
	    iog.getObjectOrderValueMap();
	for(int i=0; i<ordered.size(); i++){
	    IObject io = ordered.get(i);
	    IVector<IVectorIndex> repr = similarity.getRepresentation(io);
	    if(reductionMethod==0){
		if(i<maxNumOfObjects){
		    //		    newIOS.add(io);		    
		    //		    newIOG.add(io, maxNumOfObjects-i);
		    newIOG.add(io, objectOrderValueMap.get(io));
		}else
		    break;

	    }else if(reductionMethod==1){
		float sim = similarity.sim(repr, groupRepr);
		if(sim>minSim){
		    //		    newIOS.add(io);
		    newIOG.add(io, sim);
		}
	    }else if(reductionMethod==2){
		if(i<maxNumOfObjects){
		    float sim = similarity.sim(repr, groupRepr);
		    if(sim>minSim){
			//			newIOS.add(io);	
			newIOG.add(io, sim);
		    }
		}else
		    break;
	    }
	}

	/*
	IObjectGroup newIOG = 
	    new IObjectGroup("Reduced ("+iog.getName()+")");
	newIOG.addAll(newIOS);
	*/
	return newIOG;
    }





}
