package infomat.algorithms.reduction;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.ivector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;


/**
 * Reduces the IObjectGroup:s of a IObjectGrouping relative
 * the IObjectGroup:s of another IObjectGrouping.
 **/
public class RelativeReducer implements HasProperties{

    private static int relativeReductionNumber = 1;
    private mro.util.properties.Properties properties;

    public RelativeReducer(){
	this.properties = getDefaultProperties();
    }

    public RelativeReducer(mro.util.properties.Properties properties){
	this.properties = properties;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("Relative Reducer Properties", CurrentClassGetter.getClassName());
		
	try{

	    Property p = new Property("Reduction Method", 1, "The reduction method");

	    PropertyValueDescription pvd = new PropertyValueDescription("0", "At least \"Number of shared objects\" in common between the Relative Group and IObject.");
	    p.addValueDescription(pvd);

	    pvd = new PropertyValueDescription("1", "Similarity to the Relative Group considered as a centroid. See \"Bigger similarity to the centorid of the Relative Group than\".");
	    p.addValueDescription(pvd);

	    pvd = new PropertyValueDescription("2", "\"0\" and \"1\"");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    p = new Property("Number of shared objects", 1, "The number of objects that has to be shared between the Relative Group and the IObjects");
	    properties.addProperty(p);

	    p = new Property("Bigger similarity to the Relative Group centroid than", 0.0, "IObjects with bigger similarity to the centroid of the Relative Group are saved.");
	    properties.addProperty(p);


	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }


    /**
     * Note: does not handle missing zero-sized groups!!!
     */
    public IObjectGrouping reduceGrouping(SparseISimilarity similarity, IObjectGrouping ioging, IObjectGrouping relativeIOGING){ 
	
	if(ioging==null || relativeIOGING ==null)
	    return null;

	Vector<IObjectGroup> groups = ioging.getGroups();
	Vector<IObjectGroup> relativeGroups = relativeIOGING.getGroups();

	if(groups.size()==0 || relativeGroups.size()==0 || 
	   (groups.size() != relativeGroups.size()))
	    return null;



	Vector<IObjectGroup> newGroups = new Vector<IObjectGroup>();
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup newGroup = 
		reduceGroup(similarity, groups.elementAt(i), relativeGroups.elementAt(i));
	    newGroups.add(newGroup);
	}

	String name = "Relatively Reduced ("+ioging.getName()+") "+
	    relativeReductionNumber;
	relativeReductionNumber++;
	IObjectGrouping newIOGING = 
	    new IObjectGrouping(name);

	newIOGING.addAllGroups(newGroups);
	return newIOGING;
    }
    
    /**
     * Makes a new IObjectGroup that consists of 
     * objects that have the specified relation to the relaive group.
     */
    public IObjectGroup reduceGroup(SparseISimilarity similarity, IObjectGroup iog, IObjectGroup relativeIOG){

	int reductionMethod = (Integer) getPropertyValue("Reduction Method");
	int minNumOfSharedObjects = (Integer) 
	    getPropertyValue("Number of shared objects");
	double minSim = (Double) getPropertyValue("Bigger similarity to the Relative Group centroid than");


	IObjectGroup newIOG = 
	    new IObjectGroup("Relatively Reduced ("+iog.getName()+")");
	//	newIOG.addAll(newIOS);
	//	Vector<IObject> newIOS = new Vector<IObject>();


	IVector<IObject> relRepr = relativeIOG.getIVector();
	Vector<IObject> ordered = iog.getOrdered();
	//	HashMap<IObject, Float> objectOrderValueMap = 
	//	    iog.getObjectOrderValueMap();
	for(int i=0; i<ordered.size(); i++){
	    IObject io = ordered.get(i);
	    IVector<IVectorIndex> repr = similarity.getRepresentation(io);
	    if(reductionMethod==0){
		int num = IVector.numberOfSharedElements(repr, relRepr);
		if(num>=minNumOfSharedObjects){
		    //		    newIOS.add(io);
		    newIOG.add(io, minNumOfSharedObjects-num);
		    //		    newIOG.add(io, objectOrderValueMap.get(io));
		}
	    }else if(reductionMethod==1){
		float sim = similarity.sim(repr, relRepr);
		if(sim>minSim){
		    //		    newIOS.add(io);
		    newIOG.add(io, sim);
		}
	    }else if(reductionMethod==2){
		int num = IVector.numberOfSharedElements(repr, relRepr);
		float sim = similarity.sim(repr, relRepr);
		if(num>=minNumOfSharedObjects && sim>minSim){
		    //		    newIOS.add(io);
		    newIOG.add(io, sim);
		}
	    }
	}

	/*
	IObjectGroup newIOG = 
	    new IObjectGroup("Relatively Reduced ("+iog.getName()+")");
	newIOG.addAll(newIOS);
	*/

	return newIOG;
    }





}
