package infomat.algorithms.reduction;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.ivector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;


/**
 * Reduces the IObjectGroup:s of a IObjectGrouping 
 * to the same number of objects per IObjectGroup
 * as in another IObjectGrouping.
 **/
public class SizeReducer implements HasProperties{

    private static int reductionNumber = 1;
    private mro.util.properties.Properties properties;

    public SizeReducer(){
	this.properties = getDefaultProperties();
    }

    public SizeReducer(mro.util.properties.Properties properties){
	this.properties = properties;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("Size Reducer Properties", CurrentClassGetter.getClassName());

	try{

	    Property p = new Property("Reduction Method", 1, "The reduction method");

	    PropertyValueDescription pvd = new PropertyValueDescription("0", "Each group reduced to \"Number objects\" as in the other grouping by randomly removing some objects.");
	    p.addValueDescription(pvd);


	    pvd = new PropertyValueDescription("1", "Each group reduced to \"Number objects\" as in the other grouping in the current order.");
	    p.addValueDescription(pvd);

	    properties.addProperty(p);


	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }


    /**
     * Note: does not handle missing zero-sized groups!!!
     */
    public IObjectGrouping reduceGrouping(IObjectGrouping ioging, 
					  IObjectGrouping compareGrouping){
	
	if(ioging==null)
	    return null;

	Vector<IObjectGroup> groups = ioging.getGroups();
	Vector<IObjectGroup> compareGroups = compareGrouping.getGroups();

	if(groups.size()==0 || compareGroups.size()==0)
	    return null;

	if(groups.size() != compareGroups.size())
	    return null;

	Vector<IObjectGroup> newGroups = new Vector<IObjectGroup>();
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup newGroup = 
		reduceGroup(groups.get(i), compareGroups.get(i));
	    newGroups.add(newGroup);
	}

	String name = "Randomly Reduced ("+ioging.getName()+") "+reductionNumber;
	reductionNumber++;
	IObjectGrouping newIOGING = 
	    new IObjectGrouping(name);

	newIOGING.addAllGroups(newGroups);
	return newIOGING;
    }
    
    /**
     * Makes a new IObjectGroup that consists of 
     * a random subset of the original objects.
     */
    public IObjectGroup reduceGroup(IObjectGroup iog, 
				    IObjectGroup compareIOG){

	int reductionMethod = (Integer) getPropertyValue("Reduction Method");
	
	String groupNamePrefix = "Randomly Reduced (";
	if(reductionMethod==1)
	    groupNamePrefix = "Reduced in order (";

	IObjectGroup newIOG = 
	    new IObjectGroup(groupNamePrefix+iog.getName()+")");

	HashMap<IObject, Float> objectOrderValueMap = 
	    iog.getObjectOrderValueMap();
	Vector<IObject> ios = new Vector<IObject>(iog.getOrdered());

	if(reductionMethod==0)
	    Collections.shuffle(ios);

	for(int i=0; i<compareIOG.numberOfIObjects() && i<ios.size(); i++){
	    IObject io = ios.get(i);
	    newIOG.add(io, objectOrderValueMap.get(io));
	}
	return newIOG;
    }





}
