package infomat.evaluation;

import java.util.HashMap;
import java.util.Vector;
import java.io.*;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;


/**
 * For calculating intrinsic quality measures.
 * 
 * @version          080608
 * @author           Magnus Rosell
 */
public class IntrinsicEvaluation {

    private Properties properties;

    //    private IMatrix matrix;
    private IObjectGrouping evalGrouping;
    private ISimilarity similarity;
    private String groupingDesignation;
    private String stamp;

    private IntrinsicGroupEvaluation intrinsicGroupEvaluation = null;

    public IntrinsicEvaluation(//IMatrix matrix, 
			       IObjectGrouping evalGrouping, 
			       ISimilarity similarity, 
			       String groupingDesignation, 
			       String stamp){
	//	this.matrix = matrix;
	this.evalGrouping = evalGrouping;
	this.similarity = similarity;
	this.groupingDesignation = groupingDesignation;
	this.stamp=stamp;
	this.properties=getDefaultProperties();

	if(similarity!=null){
	    intrinsicGroupEvaluation = 
		new IntrinsicGroupEvaluation(similarity);
	}
    }

    public Properties getDefaultProperties(){

	Properties properties = 
	    new Properties("Intrinsic Evaluation Properties", CurrentClassGetter.getClassName());
		
	try{
	    Property p = new Property("Group Evaluation", true, "Evaluate groups individually");
	    PropertyValueDescription pvd = new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }

    public Measures calculateAllMeasures() 
	throws MeasureNameConflictException{
	Measures measures = new Measures("Intrinsic Measures");
	measures.addMeasure(calculateGroupingMeasures(groupingDesignation));

	/*
	if(matrix!=null)
	    measures.addMeasure(calculateBasicMatrixMeasures());
	*/

	boolean groupEvaluation = (Boolean) 
	    getPropertyValue("Group Evaluation");
	if(similarity!=null && groupEvaluation){
	    measures.addMeasure(calculateGroupMeasures());
	}

	measures.setAllMeasuresStampsRecursively(stamp);

	return measures;
    }


    public MeasureGroup allGlobalMeasuresInAGroup() 
	throws MeasureNameConflictException{

	//	System.out.println("allGlobalMeasuresInAGroup()");

	MeasureGroup mg = new Measures("Intrinsic Measures");
	mg.addMeasure(calculateGroupingMeasures(groupingDesignation));

	mg.setAllMeasuresStampsRecursively(stamp);

	//	System.out.println("allGlobalMeasuresInAGroup()");

	return mg;
    }


    // "Evaluated Grouping"
    // "Reference Grouping"
    public MeasureGroup calculateGroupingMeasures(String groupingDesignation) throws MeasureNameConflictException{

	//	System.out.println("calculateGroupingMeasures()");

	MeasureGroup mg = new MeasureGroup(groupingDesignation);
	Measure m = new Measure("Number Of IObjectGroups", evalGrouping.numberOfIObjectGroups(), "Number Of IObjectGroups in IObjectGrouping");
	mg.addMeasure(m);

	//	System.out.println("calculateGroupingMeasures() 2");


	m = new Measure("Number Of IObjects", evalGrouping.numberOfIObjects(), "Number Of IObjects in IObjectGrouping");
	mg.addMeasure(m);

	//	System.out.println("calculateGroupingMeasures() 3");


	/*
	if(matrix!=null){
	    double cellsPerObj = avgCellsPerObjInGroup(evalGrouping);
	    m = new Measure("Cells per IObject", cellsPerObj, "Number Of IMatrixCells Per IObject in the IObjectGrouping");
	    mg.addMeasure(m);
	}
	*/

	if(intrinsicGroupEvaluation!=null){
	    //	    System.out.println("calculateGroupingMeasures() intrinsicGroupEvaluation!=null");


	    m = new Measure("Self Similarity for Grouping", intrinsicGroupEvaluation.selfSimilarity(evalGrouping), "Self Similarity for Grouping considered one Group.");
	    mg.addMeasure(m);

	    //	    System.out.println("Self Similarity for Grouping");


	    m = new Measure("Weighted Avereage Self Similarity", intrinsicGroupEvaluation.weightedAvgSelfSimilarity(evalGrouping), "Average Self Similarity for Groups in Grouping (by number of IObjects).");
	    mg.addMeasure(m);

	    //	    System.out.println("Weighted Avereage Self Similarity");


	    m = new Measure("Weighted Average Grouping Similarity", intrinsicGroupEvaluation.weightedAvgGroupGroupingSimilarity(evalGrouping), "Average similarity of the groups to the whole grouping considered as one group (by number of IObjects).");
	    mg.addMeasure(m);

	    //	    System.out.println("Weighted Average Grouping Similarity");

	}

	//	    System.out.println("calculateGroupingMeasures()");

	return mg;
    }

    /*
    public MeasureGroup calculateBasicMatrixMeasures() throws MeasureNameConflictException{

	MeasureGroup mg = new MeasureGroup("Basic Matrix Measures");
	Measure m = new Measure("Number Of Row IObjects", matrix.getRowIObjectSet().numberOfIObjects(), "Number Of Row IObjects in IMatrix");
	mg.addMeasure(m);

	m = new Measure("Number Of Column IObjects", matrix.getColumnIObjectSet().numberOfIObjects(), "Number Of Column IObjects in IMatrix");
	mg.addMeasure(m);

	double cellsPerRow = avgCellsPerObjInGroup(matrix.getRowIObjectSet());
	m = new Measure("Cells per Row", cellsPerRow, "Average Number of IMatrixCells Per Row IObjects in IMatrix.");
	mg.addMeasure(m);

	double cellsPerCol = avgCellsPerObjInGroup(matrix.getColumnIObjectSet());
	m = new Measure("Cells per Column", cellsPerCol, "Average Number of IMatrixCells Per Column IObjects in IMatrix.");
	mg.addMeasure(m);


	return mg;
    }
    */

    /*
    private double avgCellsPerObjInGroup(IObjectGroup iog){
	double avg = 0.0;
	Vector<IObject> ioVec = iog.getOrdered();
	//	Vector<IObject> badOnes = new Vector<IObject>();
	for(int i=0; i<ioVec.size(); i++){
	    Vector cells = matrix.getCellsWith(ioVec.elementAt(i));

	    //	    if(cells=!null)
		avg += cells.size();
		//	    printBadOnes(iog, badOnes);
	}
	avg /= ioVec.size();
	return avg;
    }
    */



    public MeasureGroup calculateGroupMeasures() throws MeasureNameConflictException{

	MeasureGroup clusterGroup = 
	    new MeasureGroup("Intrinsic Group Measures for "+groupingDesignation);
	
	HashMap<IObjectGroup, Measure> selfSimilarityMap = null;
	HashMap<IObjectGroup, Measure> groupGroupingSimilarityMap = null;
	if(intrinsicGroupEvaluation!=null){
	    selfSimilarityMap = intrinsicGroupEvaluation.getSelfSimilarityMap(evalGrouping.getGroups());
	    groupGroupingSimilarityMap = intrinsicGroupEvaluation.getGroupGroupingSimilarityMap(evalGrouping.getGroups(), evalGrouping);
	}

	Vector<IObjectGroup> g1groups = evalGrouping.getGroups();
	for(int i = 0; i < g1groups.size(); i++){
	    IObjectGroup iog1 = g1groups.get(i);
	    MeasureGroup oneGroupGroup = 
		calculateOneGroupMeasures(iog1, i, selfSimilarityMap, groupGroupingSimilarityMap);
	    clusterGroup.addMeasure(oneGroupGroup);
	}

	clusterGroup.setAllMeasuresStampsRecursively(stamp);

	return clusterGroup;
    }

	
    private MeasureGroup calculateOneGroupMeasures(IObjectGroup iog, int groupNumber, HashMap<IObjectGroup, Measure> selfSimilarityMap, HashMap<IObjectGroup, Measure> groupGroupingSimilarityMap) throws MeasureNameConflictException{

	MeasureGroup clusterGroup = new MeasureGroup(iog.getName());
	Measure m = new Measure("Group Size", iog.numberOfIObjects(), "Group Size");
	clusterGroup.addMeasure(m);

	if(selfSimilarityMap!=null){
	    m=selfSimilarityMap.get(iog);
	    clusterGroup.addMeasure(m);
	}
	if(groupGroupingSimilarityMap!=null){
	    m=groupGroupingSimilarityMap.get(iog);
	    clusterGroup.addMeasure(m);
	}

	return clusterGroup;
    }

    /************************************/

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public Properties getProperties(){
    	return properties;
    }

    public void setProperties(Properties properties){
    	this.properties=properties;
    }

}
