package infomat.io.textresult;

import mro.util.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import infomat.vectorspace.*;
import infomat.vectorspace.ordering.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.ivector.*;

import java.io.*;
import java.util.*; 
import java.util.zip.*;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * For loading and saving groupings to text.
 * 
 * @version          080626
 * @author           Magnus Rosell
 */
public class IObjectGroupingTextResult implements HasProperties{

    private ISimilarity similarity;
    private IMatrix matrix;
    private IObjectGrouping grouping;
    //    private HashMap<IObjectGroup, IVector<IVectorIndex>> descriptionVecMap;

    private long time;

    private File resultFile;
    private String groupDirectoryName;
    private File groupDir;
    private String separator;

    private HashMap<IObjectGroup, String> groupFileNamesMap;

    private String xslGroupingFileName = "IObjectGrouping.xsl";
    private String xslGroupFileName = "IObjectGroup.xsl";

    private Properties properties;

    private IObjectFloatSorter iobjectFloatSorter = 
	new IObjectFloatSorter();


    public IObjectGroupingTextResult(){
	this.properties=getDefaultProperties();
	separator = System.getProperties().getProperty("file.separator");
    }

    public IObjectGroupingTextResult(Properties properties){
	this.properties=properties;
	separator = System.getProperties().getProperty("file.separator");
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("IObjectGrouping Text Result Properties", CurrentClassGetter.getClassName());
		
	try{
	    PropertyGroup pg = new PropertyGroup("IObjectGrouping page");

	    Property p = new Property("Number of Description IObjects For Groups", 5, "The number of Description IObjects that is presented.");
	    pg.addProperty(p);

	    p = new Property("Number of Representation IObjects For Groups", 5, "The number of Representation IObjects that is presented.");
	    pg.addProperty(p);

	    p = new Property("Number of IObjects For Groups", 3, "The number of IObjects that is presented.");
	    pg.addProperty(p);

	    p = new Property("Save hierarchy", true, "If the grouping is a hiererchy.");
	    PropertyValueDescription pvd = new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    pg.addProperty(p);

	    properties.addProperty(pg);


	    pg = new PropertyGroup("IObjectGroup pages");

	    p = new Property("Save group pages", true, "Wether the Groups should be saved.");
	    pvd = new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    pg.addProperty(p);

	    /*
	    p = new Property("Save hierarchical group pages", false, "Wether the Groups in the hierarchy should be saved.");
	    pvd = new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    pg.addProperty(p);
	    */

	    p = new Property("Number of Description IObjects For Group", 10, "The number of Description IObjects that is presented.");
	    pg.addProperty(p);

	    p = new Property("Number of Representation IObjects For Group", 10, "The number of Representation IObjects that is presented.");
	    pg.addProperty(p);

	    p = new Property("Number of IObjects For Group", 10, "The number of IObjects that is presented.");
	    pg.addProperty(p);

	    properties.addProperty(pg);

	}catch (PropertyNameConflictException pnce){
	    //			never happens
	}
	return properties;
    }


    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }

    private String fixFileName(String name){
	String ret ="";
	String[] split = name.split(" ");
	ret = split[0];
	for(int i=1; i<split.length; i++)
	    ret += "_"+split[i];
	return ret;
    }

    String iobjectNotSavedFileName = null;
    String iobjectGroupNotSavedFileName = null;
    String exportPropertiesFileName = null;
    private void saveInfoHTMLFiles(String resultDirectoryName) throws IOException{
	
	// Export Properties
	exportPropertiesFileName = fixFileName(properties.getPropertiesName());
	exportPropertiesFileName = 
	    resultDirectoryName+File.separator+exportPropertiesFileName+time+".xml";

	properties.save(exportPropertiesFileName);

	// IObject Not Saved
	iobjectNotSavedFileName = "IObjectNotSaved"+time+".html";
	String iobjectNotSavedFileNameFull = 
	    resultDirectoryName+File.separator+iobjectNotSavedFileName;
	PrintWriter out = new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(iobjectNotSavedFileNameFull))));
	String title = "IObject Not Saved";
	HTMLHelper.printHTMLHead(out, title);
	out.println("<h2>"+title+"</h2>");
	out.println("<p>The Location of the IObject is not known. There might not be any file corresponding to this IObject.</p>");
	out.println("<p>You can find the settings for the exportation of these result files");
	HTMLHelper.printHTMLLink(out, "here", exportPropertiesFileName);
	out.println(".</p>");
	HTMLHelper.printHTMLEnd(out);
	out.flush();
	out.close();

	// IObject Group Not Saved
	iobjectGroupNotSavedFileName = "IObjectGroupNotSaved"+time+".html";
	String iobjectGroupNotSavedFileNameFull = 
	    resultDirectoryName+File.separator+iobjectGroupNotSavedFileName;
	out = new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(iobjectGroupNotSavedFileNameFull))));
	title = "IObjectGroup Not Saved";
	HTMLHelper.printHTMLHead(out, title);
	out.println("<h2>"+title+"</h2>");
	out.println("<p>The Location of the IObjectGroup is not known. There might not be any file corresponding to this IObjectGroup.</p>");
	out.println("<p>You can find the settings for the exportation of these result files");
	HTMLHelper.printHTMLLink(out, "here", exportPropertiesFileName);
	out.println(".</p>");
	HTMLHelper.printHTMLEnd(out);
	out.flush();
	out.close();
    }

    private String getIObjectNotSavedFileName(){
	return iobjectNotSavedFileName;
    }

    private String getIObjectGroupNotSavedFileName(){
	return iobjectGroupNotSavedFileName;
    }

    private String getExportPropertiesFileName(){
	return exportPropertiesFileName;
    }

    /**
     * Exports IObjectGrouping in XML format
     */
    public void save(IMatrix matrix, IObjectGrouping grouping, ISimilarity similarity, File file){
	this.matrix=matrix;
	save(matrix, grouping, null, similarity, file);
    }

    /**
     * Exports IObjectGrouping in XML format
     */
    public <T extends IVectorIndex> void save(IMatrix matrix, IObjectGrouping grouping, HashMap<IObjectGroup, IVector<T>> descriptionVecMap, ISimilarity similarity, File file){
	this.matrix=matrix;
	//	this.descriptionVecMap=descriptionVecMap;
	this.grouping = grouping;
	this.similarity=similarity;
	this.resultFile = file;

	this.time = System.currentTimeMillis();


	try {
	    
	    // Info files
	    String resultDirectoryName = 
		resultFile.getCanonicalFile().getParentFile().getPath();
	    saveInfoHTMLFiles(resultDirectoryName);

	    // Group files
	    boolean saveGroupPages = (Boolean) 
		getPropertyValue("Save group pages");
	    if(saveGroupPages){
		saveGroupFiles(descriptionVecMap);
	    }

	    // Grouping xml-file
	    saveGroupingFile(descriptionVecMap);

	    // Grouping xsl-file
	    File xslFile = new File(file.getParent()+separator+xslGroupingFileName);
	    PrintWriter outxsl =
		new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(xslFile))));

	    printGroupingXSL(outxsl);
	    outxsl.close();

	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
    }


    public <T extends IVectorIndex> void saveGroupFiles(HashMap<IObjectGroup, IVector<T>> descriptionVecMap) throws IOException{
	boolean saveGroups = (Boolean)
	    getPropertyValue("Save group pages");
	if(!saveGroups)
	    return;

	String resultDirectoryName = 
	    resultFile.getCanonicalFile().getParentFile().getPath();
	
	groupDirectoryName = 
	    resultDirectoryName+separator+"groups"+time;
	groupDir = new File(groupDirectoryName);
	groupDir.mkdirs();

	groupFileNamesMap = new HashMap<IObjectGroup, String>();
	IObjectGroupTextResult iogtr = new IObjectGroupTextResult();
	try{
	    int descObjs = (Integer) 
		getPropertyValue("Number of Description IObjects For Group");
	    iogtr.setProperty("Number of Description IObjects For Group", 
			      descObjs); 
	    int reprObjs = (Integer) 
		getPropertyValue("Number of Representation IObjects For Group");
	    iogtr.setProperty("Number of Representation IObjects For Group", 
			      reprObjs); 
	    int objs = (Integer) 
		getPropertyValue("Number of IObjects For Group");
	    iogtr.setProperty("Number of IObjects For Group", objs); 
	}catch(PropertyNotFoundException pnfe){
	    // Won't happen
	}

	Vector<IObjectGroup> iogs = grouping.getGroups();
	for(int i=0; i<iogs.size(); i++){
	    IObjectGroup group = iogs.elementAt(i);
	    String groupFileName = fixFileName(group.getName());
	    groupFileName = 
		groupDirectoryName+separator+groupFileName+time+".xml";
	    //	    groupFileName = fixFileName(groupFileName);	    
	    groupFileNamesMap.put(group, groupFileName);
	    File groupFile = new File(groupFileName);
	    PrintWriter out =
		new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(groupFile))));
	    
	    /*
	    String xslGroupFileName = 
		//		resultDirectoryName+separator+this.xslGroupFileName;
		".."+separator+this.xslGroupFileName;
	    */

	    String groupLocation = groupFileName;

	    IVector<T> descriptionVec = null;
	    if(descriptionVecMap!=null){
		descriptionVec = descriptionVecMap.get(group);
	    }

	    String ioUnknownLocation = ".."+separator+iobjectGroupNotSavedFileName;


	    iogtr.saveGroupFile(out, ".."+separator+this.xslGroupFileName, 
				group, groupLocation,
				descriptionVec, similarity, 
				ioUnknownLocation);


	    // XSL
	    File xslGroupFile = new File(resultDirectoryName+separator+this.xslGroupFileName);
		//xslGroupFileName);
	    PrintWriter xslOut =
		new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(xslGroupFile))));
	    iogtr.printGroupXSL(xslOut);
	    xslOut.flush();
	    xslOut.close();



	}
    }



    public void printGroupingDTD(PrintWriter out) {
	out.println("<!DOCTYPE InfomatIObjectGroupingText [");

	out.println("<!ELEMENT InfomatIObjectGroupingText (InfomatIObjectGroupingTextGeneralInfo, IObjectGroups, Hierarchy?)>");

	out.println("<!ELEMENT InfomatIObjectGroupingTextGeneralInfo (IObjectGroupingName, TotalNumberOfIObjects, TotalNumberOfRepresentationIObjects?, NumberOfIObjectGroups)>");
	out.println("<!ELEMENT IObjectGroupingName (#PCDATA)>");

	out.println("<!ELEMENT TotalNumberOfIObjects (#PCDATA)>");
	out.println("<!ELEMENT TotalNumberOfRepresentationIObjects (#PCDATA)>");
	out.println("<!ELEMENT NumberOfIObjectGroups (#PCDATA)>");

	out.println("<!--The IObjectGroups-->");
	out.println("<!ELEMENT IObjectGroups (InfomatIObjectGroupText+)>");

	IObjectGroupTextResult.printPartGroupDTD(out);

	IVectorTextResult.printPartIVectorDTD(out);

	out.println("<!--The IObjectGroup Hierarchy-->");
	out.println("<!ELEMENT Hierarchy (IObjectGroupNode+)>");
	out.println("<!ELEMENT IObjectGroupNode (InfomatIObjectGroupText, IObjectGroupNode*)>");

	out.println("]>");
    }

    /**
     * Prints a basic XSL-file for IObjectGrouping XML-files to a 
     * {@link java.io.PrintWriter}.
     */
    public void printGroupingXSL(PrintWriter pw) throws IOException{
	pw.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
	pw.println("<xsl:stylesheet version=\"1.0\"");
	pw.println("xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\">");
	pw.println("<xsl:template match=\"/\">");
	pw.println("<html>");
	pw.println("<body>");
	pw.println("<xsl:apply-templates/>");


	pw.println("<h4>Abbreviations</h4>");
	pw.println("<table border=\"1\">");
	/*
	pw.println("<tr>");
	pw.println("<td>io</td><td>IObjects</td>");
	pw.println("</tr>");
	*/
	pw.println("<tr>");
	pw.println("<td>descr</td><td>Description IObjects</td>");
	pw.println("</tr>");
	pw.println("<tr>");
	pw.println("<td>repr</td><td>Representation IObjects</td>");
	pw.println("</tr>");
	pw.println("</table>");
	

	pw.println("</body>");
	pw.println("</html>");
	pw.println("</xsl:template>");
	pw.println("");

	pw.println("<xsl:template match=\"IObjectGroupingName\">");
	pw.println("<h2><xsl:value-of select=\".\"/></h2>");
	pw.println("</xsl:template>");

	pw.println("<xsl:template match=\"TotalNumberOfIObjects\">");
	pw.println("<xsl:value-of select=\".\"/> IObjects.");
	pw.println("</xsl:template>");
	pw.println("<xsl:template match=\"TotalNumberOfRepresentationIObjects\">");
	pw.println("<xsl:value-of select=\".\"/> RepresentationIObjects.<br /><br />");
	pw.println("</xsl:template>");

	pw.println("<xsl:template match=\"NumberOfIObjectGroups\">");
	pw.println("<b><xsl:value-of select=\".\"/> IObjectGroups: </b>");
	pw.println("<br />");
	pw.println("</xsl:template>");
	pw.println("");

	pw.println("<xsl:template match=\"IObjectGroups\">");
	pw.println("<ol>");
	pw.println("<xsl:apply-templates select=\"InfomatIObjectGroupText\"/>");
	pw.println("</ol>");
	pw.println("</xsl:template>");
	pw.println("");

	String context = ""; 
	IObjectGroupTextResult.printPartOfGroupingXSL(pw, context);

	IVectorTextResult.printPartDescriptionIVectorXSL(pw, context);
	IVectorTextResult.printPartRepresentationIVectorXSL(pw, context);
	IVectorTextResult.printPartIObjectGroupIVectorXSL(pw, context);


	// Hierarchy
	pw.println("");
	pw.println("<xsl:template match=\"Hierarchy\">");
	pw.println("<h3>Hierarchy</h3>");
	pw.println("<xsl:apply-templates select=\"IObjectGroupNode\"/>");
	pw.println("</xsl:template>");


	pw.println("<xsl:template match=\"IObjectGroupNode\">");
	pw.println("<ul>");
	pw.println("<xsl:apply-templates select=\"InfomatIObjectGroupText\"/>");
	pw.println("<xsl:apply-templates select=\"IObjectGroupNode\"/>");
	pw.println("</ul>");
	pw.println("</xsl:template>");

	pw.println("</xsl:stylesheet>");
    }





    /**
     * Prints a reference to the Properties-XSL-file to a 
     * {@link java.io.PrintWriter}.
     */
    public void printIObjectGroupingXSLReference(PrintWriter pw) throws IOException{
	pw.println("<?xml-stylesheet type=\"text/xsl\" href=\""+xslGroupingFileName+"\"?>");
    }

    private <T extends IVectorIndex> void saveGroupingFile(HashMap<IObjectGroup, IVector<T>> descriptionVecMap) throws IOException{

	PrintWriter out =
	    new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(resultFile))));

	XMLHelper.printXMLHead(out);
	printIObjectGroupingXSLReference(out); 
	printGroupingDTD(out);

	out.println("<InfomatIObjectGroupingText>");

	out.println("<InfomatIObjectGroupingTextGeneralInfo>");
	int numberOfRepresentativeObjectsForGroup = (Integer) properties.getPropertyValue("Number of Representation IObjects For Groups");

	int numberOfObjectsForGroups = (Integer) properties.getPropertyValue("Number of IObjects For Groups");

	out.println("<IObjectGroupingName>"+grouping.getName()+"</IObjectGroupingName>");
	out.println("<TotalNumberOfIObjects>"+grouping.numberOfIObjects()+"</TotalNumberOfIObjects>");
	if(similarity instanceof SparseISimilarity){
	    SparseISimilarity sis = (SparseISimilarity) similarity;
	    IVector<IVectorIndex> ivec = 
		sis.getGroupRepresentation(grouping);
	    out.println("<TotalNumberOfRepresentationIObjects>"+ivec.size()+"</TotalNumberOfRepresentationIObjects>");
	}
	out.println("<NumberOfIObjectGroups>"+grouping.numberOfIObjectGroups()+"</NumberOfIObjectGroups>");
	out.println("</InfomatIObjectGroupingTextGeneralInfo>");



	IObjectGroupTextResult iogtr = new IObjectGroupTextResult();
	try{
	    int descObjs = (Integer) 
		getPropertyValue("Number of Description IObjects For Groups");
	    iogtr.setProperty("Number of Description IObjects For Group", 
			      descObjs); 
	    int reprObjs = (Integer) 
		getPropertyValue("Number of Representation IObjects For Groups");
	    iogtr.setProperty("Number of Representation IObjects For Group", 
			      reprObjs); 
	    int objs = (Integer) 
		getPropertyValue("Number of IObjects For Groups");
	    iogtr.setProperty("Number of IObjects For Group", objs); 
	}catch(PropertyNotFoundException pnfe){
	    // Won't happen
	}

	out.println("<IObjectGroups>");
	Vector<IObjectGroup> groups = grouping.getGroups();
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup group = groups.elementAt(i);
	    String groupLocation = 
		groupDirectoryName+separator+group.getName()+time+".xml";
	    if(groupFileNamesMap!=null)
		groupLocation = groupFileNamesMap.get(group);

	    IVector<T> descriptionVec = null;
	    if(descriptionVecMap!=null){
		descriptionVec = descriptionVecMap.get(group);
	    }

	    String ioUnknownLocation = iobjectNotSavedFileName;
	    iogtr.savePartGroupFile(out, group, groupLocation, descriptionVec, similarity, ioUnknownLocation);
	}
	out.println("</IObjectGroups>");

	boolean saveHierarchy = (Boolean) getPropertyValue("Save hierarchy");

	if(saveHierarchy && grouping instanceof HierarchicalIObjectGrouping){
	    saveHierarchy(out, (HierarchicalIObjectGrouping) grouping, descriptionVecMap);
	}

	out.println("</InfomatIObjectGroupingText>");


	out.flush();
	out.close();
    }







    private <T extends IVectorIndex> void saveHierarchy(PrintWriter out, 
			       HierarchicalIObjectGrouping hging,
			       HashMap<IObjectGroup, IVector<T>> descriptionVecMap)
	throws IOException{

	out.println("<Hierarchy>");


	//	saveGroupNode(out, hging.getRootNode());
	Vector<IObjectGroupNode> nodes = hging.getRootIObjectGroupNodes();
	for(int i=0; i<nodes.size(); i++)
	    saveGroupNode(out, nodes.elementAt(i), descriptionVecMap);

	out.println("</Hierarchy>");
	
    }
	   
    private <T extends IVectorIndex> void 
		       saveGroupNode(PrintWriter out, 
				     IObjectGroupNode node,
				     HashMap<IObjectGroup, IVector<T>> descriptionVecMap) throws IOException{

	boolean saveGroupPages = (Boolean) properties.getPropertyValue("Save group pages");


	out.println("<IObjectGroupNode>");

	IObjectGroup group = node.getIObjectGroup();
	boolean leafNode = false;
	if(node.getChildren()==null)
	    leafNode=true;
	String groupLocation = group.getName();
	if(saveGroupPages && leafNode)
	    	groupLocation = groupFileNamesMap.get(group);
	else 
	    groupLocation = getIObjectGroupNotSavedFileName();

	IObjectGroupTextResult iogtr = new IObjectGroupTextResult();
	try{
	    int descObjs = (Integer) 
		getPropertyValue("Number of Description IObjects For Groups");
	    iogtr.setProperty("Number of Description IObjects For Group", 
			      descObjs); 
	    int reprObjs = (Integer) 
		getPropertyValue("Number of Representation IObjects For Groups");
	    iogtr.setProperty("Number of Representation IObjects For Group", 
			      reprObjs); 
	    //	    int objs = (Integer) 
	    //		getPropertyValue("Number of IObjects For Groups");
	    int objs = 0;
	    iogtr.setProperty("Number of IObjects For Group", objs); 
	}catch(PropertyNotFoundException pnfe){
	    // Won't happen
	}

	IVector<T> descriptionVec = null;
	if(descriptionVecMap!=null){
	    descriptionVec = descriptionVecMap.get(group);
	}
	String ioUnknownLocation = iobjectNotSavedFileName;
	iogtr.savePartGroupFile(out, group, groupLocation, descriptionVec, similarity, ioUnknownLocation);

	
	Vector<IObjectGroupNode> children = node.getChildren();
	if(children != null){
	    for(int i=0; i<children.size(); i++){
		node = children.elementAt(i);
		saveGroupNode(out, node, descriptionVecMap);
	    }
	}

	out.println("</IObjectGroupNode>");
    }


}
