package infomat.io.wordword;

import infomat.vectorspace.*;

import java.io.*;
import java.util.*; 
import java.util.zip.*;

/**
 * For reading a set of texts from a directory (and its subdirectories).
 * 
 * @version          080211
 * @author           Magnus Rosell
 */
public class WordWordDirectoryReader{

    private IMatrix matrix;
    //    private WordWordIMatrixConstructor twmc;
    private static int idCounter=1;

    public static TextFileFilter tff = new TextFileFilter();
    public static AllFileFilter aff = new AllFileFilter();

    public WordWordDirectoryReader(){
	//	this.twmc = new WordWordIMatrixConstructor();
    }

    /*
    public WordWordDirectoryReader(WordWordIMatrixConstructor twmc){
	this.twmc = twmc;
    }
    */

    /**
     * Loads a IMatrix from a directory.
     */
    public IMatrix load(File file) {
	//	matrix = new IMatrix("Words", "Texts");

	System.out.println("WordWordDirectoryReader.load()");
	    
	matrix = new IMatrix("Words", "Words");
	matrix.setColumnIObjectSet(matrix.getRowIObjectSet());


	recurseFile(file);

	matrix.getRowIObjectSet().sortInCurrentVectorOrderWithInvertedOrderNumberAsValue();
	matrix.getColumnIObjectSet().sortInCurrentVectorOrderWithInvertedOrderNumberAsValue();

	return matrix;
    }

    private void recurseFile(File file) {
	if(file.isDirectory()){
	    File[] list = file.listFiles(aff);
	    for(int i=0; i<list.length; i++){
		recurseFile(list[i]);
	    }
	}else{
	    //	    doOneFileWholeFileContext(file);
	    
	    int pre = 2;
	    int post = 2;
	    doOneFileContextWindow(file, pre, post);
	}
    }

    private int objectID = 1;
    private HashMap<String, IObject> stringWordObjectMap = 
	new HashMap<String, IObject>();

    private void doOneFileWholeFileContext(File file){

	try {

	    System.out.println(file);

	    //	    int totalLocalCount = 0;
	    //	    HashMap<IObject, Integer> wordNumberMap = 
	    //		new HashMap<IObject, Integer>();

	    HashSet<IObject> fileObjectSet = 
		new HashSet<IObject>();


	    BufferedReader in = 
		new BufferedReader(new FileReader(file));
	    String s=in.readLine();
	    while(s!=null){
		String[] words = s.split(" ");
		for(int i=0; i<words.length; i++){
		    String word = words[i];
		    //		    if(word.length()<5 || word.length()>20)
		    //			continue;

		    IObject io = stringWordObjectMap.get(word);
		    if(io==null){
			io = new IObject(words[i], objectID++);
			stringWordObjectMap.put(words[i], io);
			matrix.addColumnIObject(io);

		    }

		    if(!fileObjectSet.contains(io)){
			
			Iterator<IObject> it = fileObjectSet.iterator();
			while(it.hasNext()){
			    IObject io2 = it.next();
			    IMatrixCell mc = matrix.getCell(io, io2);
			    if(mc==null){
				mc = new IMatrixCell(io, io2, 0, 0);
				matrix.addCell(mc);
			    }
			    mc.addToLocalCount(1);
			    mc.setWeight(mc.getWeight() + 1);
			    
			    mc = matrix.getCell(io2, io);
			    if(mc==null){
				mc = new IMatrixCell(io2, io, 0, 0);
				matrix.addCell(mc);
			    }
			    mc.addToLocalCount(1);
			    mc.setWeight(mc.getWeight() + 1);
			}
			
			fileObjectSet.add(io);
		    }


		    /*
		    Integer number = wordNumberMap.get(io);
		    if(number==null){
			number=0;
		    }
		    number++;
		    wordNumberMap.put(io, number);
		    //		    totalLocalCount++;
		    */
		}

		s=in.readLine();
	    }
	    
	    in.close();



	    
		    /*	    
	    Iterator<IObject> it1 = wordNumberMap.keySet().iterator();
	    while(it1.hasNext()){
		//		IObject io1 = stringWordObjectMap.get(it1.next());
		IObject io1 = it1.next();
		Integer num1 = wordNumberMap.get(io1);

		Iterator<IObject> it2 = wordNumberMap.keySet().iterator();
		while(it2.hasNext()){
		    //		    IObject io2 = stringWordObjectMap.get(it2.next());
		    IObject io2 = it2.next();
		    Integer num2 = wordNumberMap.get(io2);
		    
		    IMatrixCell mc = matrix.getCell(io1, io2);
		    if(mc==null){
			mc = new IMatrixCell(io1, io2, 0, 0);
			matrix.addCell(mc);
		    }
		    mc.addToLocalCount(1);
		    mc.setWeight(mc.getWeight() + 1);
		    //		    mc.addToLocalCount(num1*num2);
		    //		    mc.setWeight(mc.getWeight() + num1*num2);

		    mc = matrix.getCell(io2, io1);
		    if(mc==null){
			mc = new IMatrixCell(io2, io1, 0, 0);
			matrix.addCell(mc);
		    }
		    mc.addToLocalCount(1);
		    mc.setWeight(mc.getWeight() + 1);
		    //		    mc.addToLocalCount(num1*num2);
		    //		    mc.setWeight(mc.getWeight() + num1*num2);
		}
	    }
		    */
	    

	}catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
    }

    private void printArray(String[] arr){
	for(int i=0; i<arr.length; i++)
	    System.out.print(arr[i]+", ");
	System.out.println();
	
    }


    private void doOneFileContextWindow(File file, int pre, int post){

	try {

	    System.out.println(file);

	    //	    int totalLocalCount = 0;

	    BufferedReader in = 
		new BufferedReader(new FileReader(file));
	    String s=in.readLine();
	    while(s!=null){
		String[] words = s.split(" ");
		//		System.out.print("words: ");
		//		printArray(words);

		for(int i=0; i<words.length; i++){
		    int currentI = i;
		    String[] contextWords = null;
		    if(i>=pre && words.length-i>post){
			contextWords = 
			    Arrays.copyOfRange(words, i-pre, i+post+1);
			currentI = pre;
		    }else if(i>=pre && words.length-i<=post){
			contextWords = 
			    Arrays.copyOfRange(words, i-pre, words.length);
			currentI = pre;
		    }else if(i<pre && words.length-i>post){
			contextWords = 
			    Arrays.copyOfRange(words, 0, i+post+1);
			currentI = i;
		    }else{
			contextWords = 
			    Arrays.copyOfRange(words, 0, words.length);
			currentI = i;
		    }

		    //		    System.out.print("i: "+i+", currentI: "+currentI+"   ");
		    //		    printArray(contextWords);

		    String currentWord = contextWords[currentI];
		    IObject currentIObject = 
			stringWordObjectMap.get(currentWord);
		    if(currentIObject==null){
			currentIObject = new IObject(currentWord, objectID++);
			stringWordObjectMap.put(currentWord, currentIObject);
			matrix.addColumnIObject(currentIObject);
		    }

		    for(int j=0; j<contextWords.length; j++){
			if(j==currentI)
			    continue;
			String contextWord = contextWords[j];
			IObject contextIObject = 
			    stringWordObjectMap.get(contextWord);
			if(contextIObject==null){
			    contextIObject = 
				new IObject(contextWord, objectID++);
			    stringWordObjectMap.put(contextWord, contextIObject);
			    matrix.addColumnIObject(contextIObject);
			}
	
			IMatrixCell mc = 
			    matrix.getCell(currentIObject, contextIObject);
			if(mc==null){
			    mc = new IMatrixCell(currentIObject, contextIObject, 0, 0);
			    matrix.addCell(mc);
			}
			mc.addToLocalCount(1);
			mc.setWeight(mc.getWeight() + 1);
			
		    }

		    //		    totalLocalCount++;

		}

		s=in.readLine();
	    }
	    
	    in.close();
	    
	    

	}catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
    }










}


class TextFileFilter implements FileFilter{

    public boolean accept(File pathname){
	if(pathname.isDirectory())
	    return true;
	if(pathname.getName().endsWith(".txt") ||
	   pathname.getName().endsWith(".TXT"))
	    return true;
	return false;
    }

}

class AllFileFilter implements FileFilter{

    public boolean accept(File pathname){
	return true;
    }

}
