package infomat.vectorspace;

import mro.util.*;
import mro.util.properties.*;

import java.lang.Math.*;
import java.util.*;

/**
 * For normalizeing rows or columns of a IMatrix.
 *
 * @version          080605
 * @author           Magnus Rosell
 */
public class Normalizer implements HasProperties{
    mro.util.properties.Properties properties;

    public Normalizer(){
	this.properties = getDefaultProperties();
    }

    public Normalizer(mro.util.properties.Properties properties){
	this.properties = properties;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("Normalizer Properties", CurrentClassGetter.getClassName());
		
	try{

	    // Local row weight
	    Property p = new Property("Normalize", 0, "What is normalized.");
	    PropertyValueDescription pvd = new PropertyValueDescription("0", "Rows are normalized");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("1", "Columns are normalized.");
	    p.addValueDescription(pvd);
	    /*
	    pvd = new PropertyValueDescription("2", "The matrix is normalized");
	    p.addValueDescription(pvd);
	    */
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }


    public void normalize(IMatrix matrix){ 
	int normKind = (Integer) getPropertyValue("Normalize");
	boolean row = true;
	if(normKind==1)
	    row=false;
	normalizeMatrix(matrix, row);
    }


    /**
     * Normalizes the rows or columns of the matrix.
     */
    public static void normalizeMatrix(IMatrix matrix, boolean row){
	Vector<IObject> rowObjects = 
	    matrix.getColumnIObjectSet().getOrdered();
	IObjectSet colSet = matrix.getRowIObjectSet();
	if(row){
	    rowObjects = matrix.getRowIObjectSet().getOrdered();
	    colSet = matrix.getColumnIObjectSet();
	}

	for(int i=0; i<rowObjects.size(); i++){
	    IObject rowObj = rowObjects.elementAt(i);
	    Vector cells = 
		matrix.getCellsWithIObjectRestrictedToGroup(rowObj, colSet);
	    if(cells==null)
		continue;
	    double sqsum = 0.0;
	    for(int j=0; j<cells.size(); j++){
		IMatrixCell mc = (IMatrixCell) cells.elementAt(j);
		sqsum += Math.pow(mc.getWeight(),2);
	    }

	    for(int j=0; j<cells.size(); j++){
		IMatrixCell mc = (IMatrixCell) cells.elementAt(j);
		mc.setWeight((float) Math.sqrt(Math.pow(mc.getWeight(),2)/sqsum));
	    }
	}
    }
    

}
