package infomat.vectorspace.ri;

import infomat.vectorspace.*;

import mro.util.*;
import mro.util.properties.*;

import java.io.*;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Collection;
import java.util.Map;


/**
 *
 * @version          091005
 * @author           Magnus Rosell
 */
public class RI implements HasProperties{

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = 
	    new mro.util.properties.Properties("RI Properties", CurrentClassGetter.getClassName());
		
	try{
			
	    Property p = new Property("Kind of RI", 0, "Kind of RI");
	    PropertyValueDescription pvd = new PropertyValueDescription(0, "Word-word-RI"); 
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(1, "Word-text-RI"); 
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    p = new Property("Number of Dimensions", 1800, "Number of Dimensions");
	    properties.addProperty(p);


	    p = new Property("Kind of Random Labels", 0, "Kind of Random Labels"); 
	    pvd = new PropertyValueDescription(0, "Equal Number of plus ones and minus ones");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    p = new Property("Number of plus ones and minus ones", 4, "Number of plus ones and minus ones. Used when \"Kind of Random Labels\" = 0, i.e. \"Equal Number of plus ones and minus ones\"");
	    properties.addProperty(p);



	    p = new Property("Word Window Weighting Scheeme", "infomat.vectorspace.ri.ConstantWordWindowWeighting", "Word Window Weighting Scheeme");
	    pvd = new PropertyValueDescription("infomat.vectorspace.ri.ConstantWordWindowWeighting", "Constant Weighting");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    p = new Property("Left Window Size", 2, "Left Window Size");
	    properties.addProperty(p);

	    p = new Property("Right Window Size", 2, "Right Window Size");
	    properties.addProperty(p);


			
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }

    protected Properties properties;

    private IObjectSet objectSet;
    private IObjectSet indexSet;

    private HashMap<String, float[]> objectReprMap = 
	new HashMap<String, float[]>();

    private HashMap<String, RISparseVector> objectIndexMap = 
	new HashMap<String, RISparseVector>();

    private int dimensionality;
    private int nr;

    public RI(){ //IObjectSet objectSet, IObjectSet indexSet){
	this.properties = getDefaultProperties();	

	/*
	objectSet = new IObjectSet();
	indexSet = new IObjectSet();
	*/

	dimensionality = (Integer) properties.getPropertyValue("Number of Dimensions");
	nr = (Integer) properties.getPropertyValue("Number of plus ones and minus ones");


    }

    public RISparseVector addNewWord(String newWord){//, int n, int nr){
	

	RISparseVector risv = objectIndexMap.get(newWord);
	if(risv!=null)
	    return risv;

	boolean success = false;
	while(!success){
	    success = true;
	    risv = new RISparseVector(dimensionality, nr);
	    Iterator<RISparseVector> it = 
		objectIndexMap.values().iterator();
	    while(it.hasNext()){
		RISparseVector otherRISV = it.next(); 
		if(risv.equals(otherRISV)){
		    success = false;
		    break;
		}
	    }

	}
	
	objectIndexMap.put(newWord, risv);

	//	System.out.println(" --> "+newWord);
	//	System.out.println(risv);

	return risv;

	    /*
	    Iterator<Map.Entry<String, RISparseVector>> it = 
		objectIndexMap.iterator();
	    while(it.hasNext()){
		Map.Entry<String, RISparseVector> entry = it.next(); 
		
	    }
	    */	
    }

    public void add(String focusWord, String contextWord, float weight){
	float[] focusRepr = objectReprMap.get(focusWord);
	if(focusRepr==null){
	    focusRepr = new float[dimensionality];
	    objectReprMap.put(focusWord, focusRepr);
	}
	RISparseVector contextRISV = objectIndexMap.get(contextWord);
	contextRISV.addWeightedToArray(focusRepr, weight);
    }

    
    public void normalizeRepresentationVectors(){
	Iterator<Map.Entry<String, float[]>> it = 
	    objectReprMap.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<String, float[]> entry = it.next();
	    


	}
    }


    
    public float[] getRepresentation(String string){
	return objectReprMap.get(string);
    }

    public RISparseVector getIndex(String string){
	return objectIndexMap.get(string);
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }


}
