package infomat.view;

import infomat.algorithms.grouping.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.vectorspace.ri.RI;
import infomat.vectorspace.similarity.*;
import infomat.view.algorithms.*;
import infomat.view.tools.*;
import infomat.view.grouping.settings.*;
import infomat.view.pixel.*;
import infomat.view.sorter.*;
import infomat.view.similarity.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import javax.swing.*;


/**
 * 
 *
 * @version          081031
 * @author           Magnus Rosell
 */
public class GUIIMatrix{

    private IMatrix matrix;
    private GUIGroupingsForGroup rowGroupings;
    private GUIGroupingsForGroup colGroupings;


    public GUIIMatrix(IMatrix ri){
	this.matrix = ri;
	rowGroupings = 
	    new GUIGroupingsForGroup(ri.getRowIObjectSet());
	colGroupings = 
	    new GUIGroupingsForGroup(ri.getColumnIObjectSet());
    }

    public void printGroupings(){
	System.out.println("GUIIMatrix - groupings");
	System.out.println("rowGroupings: "+rowGroupings.getGroupings().size());
	System.out.println("colGroupings: "+rowGroupings.getGroupings().size());
    }
    

    public IMatrix getIMatrix(){
	return matrix;
    }

    public GUIGroupingsForGroup getRowGroupings(){
	return rowGroupings;
    }

    public GUIGroupingsForGroup getColumnGroupings(){
	return colGroupings;
    }

    public GUIGroupingsForGroup getGroupings(boolean row){
	if(row)
	    return getRowGroupings();
	return getColumnGroupings();
    }

    public IObjectGrouping getCurrentRowGrouping(){
	return rowGroupings.getCurrentGrouping();
    }

    public IObjectGrouping getCurrentColumnGrouping(){
	return colGroupings.getCurrentGrouping();
    }


    public IObjectGrouping getCurrentGrouping(boolean row){
	if(row)
	    return rowGroupings.getCurrentGrouping();
	return colGroupings.getCurrentGrouping();
    }

    public void purgeMatrixToCurrentGroupings(){
	matrix.purgeMatrixToGroups(rowGroupings.getCurrentGrouping(),
				   colGroupings.getCurrentGrouping());
	//	rowGroupings.purgeToCurrentGrouping();
	//	colGroupings.purgeToCurrentGrouping();
	rowGroupings.purgeGroupingsToGroup();
	colGroupings.purgeGroupingsToGroup();
    }


    public void invertMatrix(){
	matrix.invertMatrix();
	GUIGroupingsForGroup tmpGroupings = rowGroupings;
	rowGroupings = colGroupings;
	colGroupings = tmpGroupings;
    }

    /**
     * Resorts the IObjectSet:s "rows" and "cols" according to their
     * current IObjectGroupings.
     */
    public void resort(){
	rowGroupings.getCurrentGrouping().sort();
	colGroupings.getCurrentGrouping().sort();
    }

    public void removeColumns(int colFrom, int colTo){
	getCurrentColumnGrouping().remove(colFrom, colTo);
    }


    public void removeRows(int rowFrom, int rowTo){
	getCurrentRowGrouping().remove(rowFrom, rowTo);
    }

    
    public void removeStopIObjects(Stoplist stoplist, boolean row){
	stoplist.removeStopIObjects(matrix, row);
	if(row)
	    rowGroupings.purgeGroupingsToGroup();
	else
	    colGroupings.purgeGroupingsToGroup();
    }		


    public Dimension getCurrentSize(){
	return new Dimension(getCurrentColumnGrouping().numberOfIObjects(), 
			     getCurrentRowGrouping().numberOfIObjects());
    }


}
