package infomat.view.algorithms;

import infomat.algorithms.description.*;
import infomat.algorithms.reduction.*;
import infomat.algorithms.reduction.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.ivector.*;
import infomat.view.*;
import infomat.view.util.*;
import infomat.view.grouping.settings.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;

import javax.swing.*;


	
/**
 *
 *
 * @version          090104
 * @author           Magnus Rosell
 *
 */
public class DescriptionFrame extends JFrame
    implements ActionListener{

    public Dimension DESCRIPTIONFRAME_PREFERRED_SIZE = 
	new Dimension(350, 600);

    private JPanel generalReducerPanel;

    private DescriberSelectionPanel describerSelectionPanel;
    private DescriptionGrouper descriptionGrouper;

    private Describer describer;
    private Reducer reducer;
    private RelativeReducer relativeReducer;
	
    private InfomatModel model;
    private InfomatView view;
		
		
    private AlternatingButton rowsOrColumns;
    private AlternatingButton useReducerB;
    private AlternatingButton useRelativeReducerB;
    private JButton apply;
    private JButton cancel;
		


    public DescriptionFrame(InfomatModel model, InfomatView view){
	super("Description");

	this.model=model;
	this.view=view;
			
	if(model.getIMatrix()==null || view==null || model==null){
	    dispose();
	    return;
	}

	descriptionGrouper = new DescriptionGrouper();

	describerSelectionPanel = new DescriberSelectionPanel(this);
	describer = describerSelectionPanel.getSelectedDescriber();

	String[] alternatives = {"Rows", "Columns"};
	rowsOrColumns = new AlternatingButton(alternatives);
	rowsOrColumns.addActionListener(this);

	String[] redAlt = {"True", "False"};
	useReducerB = new AlternatingButton(redAlt);
	useReducerB.addActionListener(this);

	useRelativeReducerB = new AlternatingButton(redAlt);
	useRelativeReducerB.addActionListener(this);

	setVisible(true);
	setPreferredSize(DESCRIPTIONFRAME_PREFERRED_SIZE);

	
	reducer = new Reducer();
	relativeReducer = new RelativeReducer();

	generalReducerPanel = new JPanel();
	generalReducerPanel.setPreferredSize(DESCRIPTIONFRAME_PREFERRED_SIZE);
	generalReducerPanel.setLayout(new BoxLayout(generalReducerPanel, BoxLayout.Y_AXIS));



	putAll();
	add(generalReducerPanel);
	pack();
    }

    public void putAll(){
	//	removeAll();
	generalReducerPanel.removeAll();
	
	Container container = getContentPane();

	JTextField dummy = new JTextField("Apply");
	JButton dummyButton = new JButton("Cancel");
	int height = (int) dummy.getPreferredSize().getHeight();
	int width = (int) dummyButton.getPreferredSize().getWidth();
	Dimension buttonSize = new Dimension(width, height);
			

	JPanel infoAndButtons = new JPanel();
	infoAndButtons.setLayout(new GridLayout(5,1));
	JLabel info = new JLabel("Alter properties below and press \"Apply\"");
	infoAndButtons.add(info);
			
	JPanel dimPanel = new JPanel();
	dimPanel.setLayout(new BoxLayout(dimPanel, BoxLayout.X_AXIS));
	JLabel dimInfo = new JLabel("Description for rows or columns: ");
	dimPanel.add(dimInfo);

	rowsOrColumns.setPreferredSize(buttonSize);
	dimPanel.add(rowsOrColumns);
	infoAndButtons.add(dimPanel);

	JPanel redPanel = new JPanel();
	redPanel.setLayout(new BoxLayout(redPanel, BoxLayout.X_AXIS));
	JLabel redInfo = new JLabel("Use Reducer: ");
	redPanel.add(redInfo);
	useReducerB.setPreferredSize(buttonSize);
	redPanel.add(useReducerB);
	infoAndButtons.add(redPanel);

	redPanel = new JPanel();
	redPanel.setLayout(new BoxLayout(redPanel, BoxLayout.X_AXIS));
	redInfo = new JLabel("Use Relative Reducer: ");
	redPanel.add(redInfo);
	useRelativeReducerB.setPreferredSize(buttonSize);
	redPanel.add(useRelativeReducerB);
	infoAndButtons.add(redPanel);

			
	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
	apply = new JButton("Apply");
	apply.setPreferredSize(buttonSize);
	apply.addActionListener(this);
	buttonPanel.add(apply);
	cancel = new JButton("Cancel");
	cancel.setPreferredSize(buttonSize);
	cancel.addActionListener(this);
	buttonPanel.add(cancel);
	infoAndButtons.add(buttonPanel);
	infoAndButtons.setBorder(new TitledBorder("Description"));
	generalReducerPanel.add(infoAndButtons);
			

	JTabbedPane pane = new JTabbedPane();

			
	String path = (String) model.getPropertyValue("Result Path");


	JPanel describerPanel = new JPanel();
	describerPanel.setLayout(new BoxLayout(describerPanel, BoxLayout.Y_AXIS));


	describerPanel.add(describerSelectionPanel);

	PropertiesPanel propertiesPanel = new PropertiesPanel(describer, path);
	describerPanel.add(propertiesPanel);
	


	pane.add("Describer", describerPanel);


	propertiesPanel = new PropertiesPanel(reducer, path);
	pane.add("Reducer", propertiesPanel);

	propertiesPanel = new PropertiesPanel(relativeReducer, path);
	pane.add("Relative Reducer", propertiesPanel);

	generalReducerPanel.add(pane); //ropertiesPanel);
			
	//setSize(REDUCTIONFRAME_PREFERRED_SIZE);
	//setMinimumSize(REDUCTIONFRAME_PREFERRED_SIZE);
	//	pack();

	//	add(generalReducerPanel);
	//	pack();
	generalReducerPanel.revalidate();
	generalReducerPanel.repaint();
	repaint();
    }
		
    public void actionPerformed(ActionEvent e) {

	if(e.getSource() instanceof JButton){
	    JButton source = (JButton) e.getSource();
	    if(source == rowsOrColumns){
		rowsOrColumns.press();
		generalReducerPanel.repaint();
		repaint();
	    }else if(source == useReducerB){
		useReducerB.press();
		generalReducerPanel.repaint();
		repaint();
	    }else if(source == useRelativeReducerB){
		useRelativeReducerB.press();
		generalReducerPanel.repaint();
		repaint();
	    }else if(source == apply){
		
		boolean row=true;
		if(rowsOrColumns.getText().equals("Columns"))
		    row=false;
		
		IObjectGrouping ioging = 
			model.getGUIIMatrix().getCurrentGrouping(row);
		ISimilarity sim = model.getISimilarity(row);
		if(sim instanceof SparseISimilarity){
		    SparseISimilarity ssim = 
			(SparseISimilarity) sim;


		    Describer describer = 
			describerSelectionPanel.getSelectedDescriber();
		    
		    HashMap<IObjectGroup, IVector<IVectorIndex>> groupingDescription = 
			describer.describeGrouping(ioging, ssim);

		    if(groupingDescription==null)
			return;

		    IObjectGrouping clustering = descriptionGrouper.groupingFromDescription(ioging, groupingDescription);

		    System.out.println("Clustering: "+clustering.numberOfIObjectGroups()+" groups, "+clustering.numberOfIObjects()+" objects ");


		    // Reduction 
		    if(useReducerB.getText().equals("True")){
			SparseISimilarity similarity = 
			    model.getSimilarityManager().getSimilarity(row);
			clustering = 
			    reducer.reduceGrouping(ssim, clustering);
			clustering.removeEmptyGroups();
		    }

		    clustering.sort();

		    // Relative reduction
		    if(useRelativeReducerB.getText().equals("True")){
			IObjectGrouping relativeIOGING = clustering;
			if(ioging.numberOfIObjectGroups() != 
			   relativeIOGING.numberOfIObjectGroups()){
			    System.out.println("Relative Reduction: Not the same number of groups in groupings!");
			    return;
			}

			IObjectGrouping ioging2 = 
			    relativeReducer.reduceGrouping(ssim, ioging, relativeIOGING);
			ioging2.removeEmptyGroups();

			ioging2.sort();

			GUIIMatrix guimatrix = model.getGUIIMatrix();
			guimatrix.getGroupings(row).setGrouping(ioging2);
			view.getGroupingManager(row).putNewGrouping(ioging2);
		    }
  

		    GUIIMatrix guimatrix = model.getGUIIMatrix();
		    guimatrix.getGroupings(!row).setGrouping(clustering);
		    view.getGroupingManager(!row).putNewGrouping(clustering);
		    view.validateIMatrixBorders();
		    
		    view.forceChange();
		    view.getOverview().forceChange();
		    
		    model.update();
		}


		/*
		String reductionType = "Reduction";
		//		    reductionChoiceB.getText();
		
		if(reductionType.equals("Reduction")){

		    ProgressFrame pf = 
			new ProgressFrame("Reduction");
		    
		    IObjectGrouping ioging = 
			model.getGUIIMatrix().getCurrentGrouping(row);
		    
		    SparseISimilarity similarity = 
			model.getSimilarityManager().getSimilarity(row);
		    
		    IObjectGrouping clustering = 
			reducer.reduceGrouping(similarity, ioging);
		    clustering.removeEmptyGroups();
		    
		    GUIIMatrix guimatrix = model.getGUIIMatrix();
		    guimatrix.getGroupings(row).setGrouping(clustering);
		    view.getGroupingManager(row).putNewGrouping(clustering);
		    view.validateIMatrixBorders();

		    view.forceChange();
		    view.getOverview().forceChange();
		    
		    
		    model.update();
		    pf.dispose();
		    
		}else if(reductionType.equals("Relative Reduction")){
		    
		    ProgressFrame pf = 
			new ProgressFrame("Relative Reduction");
		    
		    IObjectGrouping ioging = 
			model.getGUIIMatrix().getCurrentGrouping(row);
		    IObjectGrouping relativeIOGING = 
			model.getGUIIMatrix().getCurrentGrouping(!row);
		    if(ioging.numberOfIObjectGroups() != 
		       relativeIOGING.numberOfIObjectGroups()){
			pf.dispose();
			System.out.println("Relative Reduction: Not the same number of groups in groupings!");
			return;
		    }
		    
		    
		    SparseISimilarity similarity = 
			model.getSimilarityManager().getSimilarity(row);
		    
		    IObjectGrouping clustering = 
			relativeReducer.reduceGrouping(similarity, ioging, relativeIOGING);
		    clustering.removeEmptyGroups();
		    
		    GUIIMatrix guimatrix = model.getGUIIMatrix();
		    guimatrix.getGroupings(row).setGrouping(clustering);
		    view.getGroupingManager(row).putNewGrouping(clustering);
		    view.validateIMatrixBorders();

		    view.forceChange();
		    view.getOverview().forceChange();
		    
		    
		    model.update();
		    pf.dispose();
		}
		*/
 
	    }else if(source == cancel){
		setVisible(false);
	    }
		   
		   
	}
    }
    

    public void setDescriber(Describer describer){
	this.describer = describer;
	putAll();
    }


    public void paint(Graphics g){
	super.paint(g);
    }
    
		
}

    
	
