package infomat.view.similarity;

import infomat.algorithms.grouping.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.view.*;
import infomat.view.util.*;
import infomat.view.grouping.settings.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;

import javax.swing.*;


	
/**
 *
 * @version          080811
 * @author           Magnus Rosell
 */
public class SimilarityFrame extends JFrame
    implements Observer{

    public static Dimension SIMILARITYFRAME_PREFERRED_SIZE = 
	new Dimension(700, 600);

    private InfomatModel model;
    private SimilarityManager similarityManager;


    private JPanel mainPanel;
    private boolean visible=true;
    private SimilarityPanel rowSimPanel;
    private SimilarityPanel colSimPanel;


    public SimilarityFrame(InfomatModel model){
	super("Similarity");//algorithm.getAlgorithmName());
	this.model=model;
	
	model.addObserver(this);

	similarityManager = new SimilarityManager(model);

	setVisible(visible);
	setPreferredSize(SIMILARITYFRAME_PREFERRED_SIZE);

	Container container = getContentPane();
	mainPanel = new JPanel();
	container.add(mainPanel);

	putAll();
	pack();
    }

    public void putAll(){
	mainPanel.removeAll();
	mainPanel.setLayout(new GridLayout(1,2));

	boolean row=true;
	rowSimPanel = 
	    new SimilarityPanel(similarityManager, model, row);
	JPanel rowPanel = new JPanel();
	rowPanel.add(rowSimPanel);
	rowPanel.setBorder(new TitledBorder("Row Similarity"));

	row=false;
	colSimPanel = 
	    new SimilarityPanel(similarityManager, model, row);
	JPanel colPanel = new JPanel();
	colPanel.add(colSimPanel);
	colPanel.setBorder(new TitledBorder("Column Similarity"));

	mainPanel.add(rowPanel);
	mainPanel.add(colPanel);
	mainPanel.revalidate();
	repaint();
    }


    public SparseISimilarity getISimilarity(boolean row){
	return similarityManager.getSimilarity(row);
	/*
	if(row)
	    return rowSimPanel.getSelectedSimilarity();
	return colSimPanel.getSelectedSimilarity();
	*/
    }

    public SimilarityManager getSimilarityManager(){
	return similarityManager;
    }

    public void invert(){
	similarityManager.invert();
	putAll();
    }

  
    /** Called by the Observable */
    public void update(Observable o, Object arg) {
	rowSimPanel.putAll();
	colSimPanel.putAll();
    }


    public void paint(Graphics g){
	super.paint(g);
    }

    public void toggleVisible(){
	visible=!visible;
	super.setVisible(visible);
    }

 
}

