package infomat.view.sorter;

import infomat.evaluation.ExtrinsicGroupEvaluation;
import infomat.evaluation.IntrinsicGroupEvaluation;
import infomat.vectorspace.IObjectGroup;
import infomat.vectorspace.IObjectGrouping;
import infomat.vectorspace.IObjectSet;
import infomat.vectorspace.MeasureSorter;
import infomat.vectorspace.similarity.*;
import infomat.view.*;

import mro.util.Round;
import mro.util.measures.*;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.*;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.border.LineBorder;
import javax.swing.filechooser.FileFilter;


/**
 * Displayer of sorter panel info. 
 * Here you can sort groupings according to different measures.
 *
 */
public class SorterPanel extends JPanel 
    implements Observer, ActionListener{

    private InfomatModel model;
    private InfomatView view;
    private IObjectGrouping currentIoging;
    private boolean rowPanel;

    private JTextField groupingNameTextField;

    private GridBagConstraints gbc;

    private ChangeGroupingPanel changeGroupingPanel;
    private GroupingInfo ginfo;
    private SortingChoicePanel sortingChoicePanel;
    private SorterMethodPanel sorterMethodPanel;
    private GroupsHeader gheader;
    private Groups groups;
    private HashMap<IObjectGroup, Measure> groupMeasureMap;

    private static int textHeight = 20;
    private static Dimension panelDimension = new Dimension(300, 500);

    public SorterPanel(InfomatModel model, InfomatView view,
		       boolean rowPanel){
	//super(new GridLayout(3,1));
	//super(new GridBagLayout());
	//gbc = new GridBagConstraints();
	//gbc.fill = GridBagConstraints.VERTICAL;
	//gbc.anchor = GridBagConstraints.LINE_START;

	//	super(new GridLayout(1,2));
	//	setPreferredSize(new Dimension(800, 450));
	this.model=model;
	this.view=view;
	this.rowPanel=rowPanel;

	this.currentIoging=
	    model.getGUIIMatrix().getCurrentGrouping(rowPanel);

	this.gheader=new GroupsHeader(null);

	this.changeGroupingPanel = new ChangeGroupingPanel(this, model, "Rows");

	this.ginfo = new GroupingInfo(currentIoging, model, rowPanel);
	this.groups=new Groups(currentIoging, rowPanel, groupMeasureMap);
	model.addObserver(this);

	setPreferredSize(panelDimension);

	this.sortingChoicePanel = new SortingChoicePanel(this);

	this.sorterMethodPanel = new ExtrinsicGroupMeasureSorterPanel(this, model, rowPanel, sortingChoicePanel.getSelected());

	putAll();
    }

    public void paintComponent(Graphics g) {
	super.paintComponent(g);
    }

    public void putAll(){
	removeAll();
	// Grouping
	IObjectGrouping grouping=
	    model.getGUIIMatrix().getCurrentGrouping(rowPanel);
	if(grouping==null){
	    add(new Label("No Grouping..."));
	}
	else{
	    //			GroupingInfo ginfo = new GroupingInfo(grouping, model, ios);
	    //ginfo.setLayout(new BoxLayout(ginfo, BoxLayout.Y_AXIS));
	    add(ginfo);




	    add(sortingChoicePanel);
	    add(sorterMethodPanel);

	    //			Panel containing Groups Header and Groups, within a frame

	    JPanel groupsPanel = new JPanel();
	    groupsPanel.setBorder(new LineBorder(Color.BLACK));
	    groupsPanel.setLayout(new BoxLayout(groupsPanel, BoxLayout.Y_AXIS));

	    //Groups Header
	    groupsPanel.add(gheader);

	    // Groups
	    //			groups = new Groups(grouping, rowPanel, groupMeasureMap);

	    groupsPanel.add(groups);

	    add(groupsPanel);

	    // Ordering
	    Ordering ordering = new Ordering(model, view, this);
	    add(ordering);



	    //			add(changeGroupingPanel);

	}

	JButton dummy = new JButton("Hej!");
	Dimension d = new Dimension((int) panelDimension.getWidth(), (int) dummy.getPreferredSize().getHeight());//(int)panelDimension.getHeight()/5);

	revalidate();
	repaint();
    }


    public void actionPerformed(ActionEvent e) {
	if(e.getSource()==sorterMethodPanel.getCalculationButton()){ 
	    ExtrinsicGroupMeasureSorterPanel egmsp = (ExtrinsicGroupMeasureSorterPanel) sorterMethodPanel;
	    IObjectGrouping ioging = egmsp.getCurrentGrouping();


	    if(egmsp.getMeasureName().equals("Purity")){
		ExtrinsicGroupEvaluation extrinsicGroupEvaluation=
		    new ExtrinsicGroupEvaluation(currentIoging, ioging);
		groupMeasureMap = 
		    extrinsicGroupEvaluation.getGroupPurityMap();

	    }else if(egmsp.getMeasureName().equals("Entropy")){
		ExtrinsicGroupEvaluation extrinsicGroupEvaluation=
		    new ExtrinsicGroupEvaluation(currentIoging, ioging);
		groupMeasureMap = 
		    extrinsicGroupEvaluation.getGroupEntropyMap();

	    }else if(egmsp.getMeasureName().equals("Part Purity")){
		float[] groupSplitValues = {0.1f, 0.2f, 0.4f, 0.6f, 0.8f};
		groupMeasureMap = 
		    ExtrinsicGroupEvaluation.getGroupPartPurityMap(currentIoging, ioging, groupSplitValues);
	    }else if(egmsp.getMeasureName().equals("Self Similarity")){
		boolean row = true;
		ISimilarity similarity = model.getISimilarity(row);
		IntrinsicGroupEvaluation ige = new IntrinsicGroupEvaluation(similarity);
		Vector<IObjectGroup> groups = currentIoging.getGroups();
		groupMeasureMap = ige.getSelfSimilarityMap(groups);

		/*
		//HashMap<IObjectGroup, Measure>
		Iterator<Map.Entry<IObjectGroup, Measure>> it = groupMeasureMap.entrySet().iterator();
		while(it.hasNext()){
		    Map.Entry<IObjectGroup, Measure> entry = it.next();
		    System.out.println(entry.getKey().getString()+": "+entry.getValue().getValue());
		}
		*/

	    }

	    gheader = new GroupsHeader(sorterMethodPanel);
	    //			groups = new Groups(ios.getCurrentGrouping(), rowPanel, groupMeasureMap);
	    groups = new Groups(currentIoging, rowPanel, groupMeasureMap);
	    putAll();
	}
	else if(e.getSource()==changeGroupingPanel.getRowsOrColumnsButton()){ 
	    JButton rc = (JButton) e.getSource();
	    if(rc.getText().equals("Columns")){
		rc.setText("Rows");
		rowPanel = true;
		//				changeGroupingPanel.setNewGroupsInComboBox(model.getIMatrix().getColumnIObjectSet());
		changeGroupingPanel = new ChangeGroupingPanel(this, model, "Rows");
		sorterMethodPanel=new ExtrinsicGroupMeasureSorterPanel(this, model, rowPanel, sortingChoicePanel.getSelected());
	    }
	    else if(rc.getText().equals("Rows")){
		rc.setText("Columns");
		rowPanel = false;
		//				changeGroupingPanel.setNewGroupsInComboBox(model.getIMatrix().getRowIObjectSet());
		changeGroupingPanel = new ChangeGroupingPanel(this, model, "Columns");
		sorterMethodPanel=new ExtrinsicGroupMeasureSorterPanel(this, model, rowPanel, sortingChoicePanel.getSelected());
	    }

	    putAll();
	}
	else if(e.getSource()==changeGroupingPanel.getApplyButton()){ 
	    String selected = changeGroupingPanel.getSelected();
	    currentIoging = model.getGUIIMatrix().getGroupings(rowPanel).getGroupingWithName(selected);

	    ginfo = new GroupingInfo(currentIoging, model, rowPanel);

	    gheader = new GroupsHeader(sorterMethodPanel);
	    groups = new Groups(currentIoging, rowPanel, groupMeasureMap);

	    putAll();

	}

	else if(e.getSource() instanceof JComboBox){ 
	    JComboBox cBox=(JComboBox) e.getSource();

	    ExtrinsicGroupMeasureSorterPanel egmsp = (ExtrinsicGroupMeasureSorterPanel) sorterMethodPanel;
	    if(cBox==egmsp.getComboBox())
		egmsp.setMeasureName(sortingChoicePanel.getSelected());
	    else if(cBox==sortingChoicePanel.getComboBox()){
		//obs mste hantera andra evalueringar sen
		String refGroupingName = (String)((ExtrinsicGroupMeasureSorterPanel)sorterMethodPanel).getComboBox().getSelectedItem();
		sorterMethodPanel=new ExtrinsicGroupMeasureSorterPanel(this, model, rowPanel, sortingChoicePanel.getSelected());
		((ExtrinsicGroupMeasureSorterPanel)sorterMethodPanel).getComboBox().setSelectedItem(refGroupingName);
		//				sorterMethodPanel = new ExtrinsicGroupMeasureSorterPanel(this, ios, sortingChoicePanel.getSelected());
		((ExtrinsicGroupMeasureSorterPanel)sorterMethodPanel).setMeasureName(sortingChoicePanel.getSelected());

	    }
	    else if(cBox==changeGroupingPanel.getChoiceComboBox()){
	    }
	    putAll();
	}


    }

    public IObjectGrouping getCurrentGrouping(){
	return model.getGUIIMatrix().getCurrentGrouping(rowPanel);
    }

    public HashMap<IObjectGroup, Measure> getGroupMeasureMap(){
	return groupMeasureMap;
    }

    /** Called by the Observable */
    public void update(Observable o, Object arg) {
	putAll();
	revalidate();
	repaint();
    }


    /***************************************************/
    private class ChangeGroupingPanel extends JPanel{
	protected JButton rowsOrColumns;
	protected JButton apply;
	protected JLabel groupChoice;

	protected InfomatModel model;

	protected JComboBox cBox;

	public ChangeGroupingPanel(SorterPanel sorterPanel, InfomatModel model, String rowOrColumn){
	    this.model=model;
	    //			setBorder(new TitledBorder("Change grouping"));
	    setBorder(new LineBorder(Color.BLACK));
	    groupChoice = new JLabel("Change group: ");
	    groupChoice.setAlignmentX(CENTER_ALIGNMENT);

	    JTextField dummyTextField = new JTextField("Columns");
	    int height = (int) dummyTextField.getPreferredSize().getHeight();
	    JButton dummy = new JButton("Columns");

	    int pheight = (int) (this.getPreferredSize().getHeight());
	    //			int pwidth = (int) (sorterPanel.getPreferredSize().getWidth());
	    int pwidth = 300;
	    //			Dimension pd = new Dimension(pwidth, height*4);
	    Dimension pd = new Dimension((int)panelDimension.getWidth(), height*4);
	    setPreferredSize(pd);
	    setMinimumSize(pd);
	    setMaximumSize(pd);

	    JLabel l = new JLabel("Choose new grouping to sort:");
	    add(l);

	    JPanel choice = new JPanel();
	    choice.setLayout(new BoxLayout(choice, BoxLayout.X_AXIS));
	    this.rowsOrColumns=new JButton(rowOrColumn);
	    choice.add(rowsOrColumns);

	    int width = (int) dummy.getPreferredSize().getWidth();
	    rowsOrColumns.setPreferredSize(new Dimension(width, height));

	    //			rowsOrColumns.setAlignmentX(CENTER_ALIGNMENT);

	    rowsOrColumns.addActionListener(sorterPanel);

	    boolean row = true;
	    if(rowsOrColumns.getText().equals("Columns"))
		row = false;

	    Vector<IObjectGrouping> groupings = 
		model.getGUIIMatrix().getGroupings(row).getGroupings();
	    if(groupings != null && groupings.size()>0){
		Vector<String> strings = new Vector<String>();

		for(int i=0; i<groupings.size(); i++)
		    strings.add(groupings.elementAt(i).getName());

		cBox = new JComboBox(strings);

		JTextField dummyField = new JTextField("Initial Grouping for Documents");
		int dummyWidth = (int) dummyField.getPreferredSize().getWidth();
		//				Dimension d = new Dimension((int)cBox.getPreferredSize().getWidth(), height);
		Dimension d = new Dimension(dummyWidth, height);

		cBox.setPreferredSize(d);
		cBox.setMinimumSize(d);
		cBox.setMaximumSize(d);
	    }
	    choice.add(cBox);
	    add(choice);
	    apply = new JButton("Apply");
	    apply.setPreferredSize(new Dimension(width, height));
	    apply.addActionListener(sorterPanel);
	    add(apply);

	}
	public JButton getRowsOrColumnsButton(){
	    return rowsOrColumns;
	}

	public JButton getApplyButton(){
	    return apply;
	}

	public JComboBox getChoiceComboBox(){
	    return cBox;
	}

	public String getSelected(){
	    return (String) cBox.getSelectedItem();
	}

	public void setNewGroupsInComboBox(IObjectSet ios){
	    boolean row = true;
	    if(rowsOrColumns.getText().equals("Columns"))
		row = false;

	    Vector<IObjectGrouping> groupings = 
		model.getGUIIMatrix().getGroupings(row).getGroupings();
	    if(groupings != null && groupings.size()>0){
		Vector<String> strings = new Vector<String>();

		for(int i=0; i<groupings.size(); i++){
		    strings.add(groupings.elementAt(i).getName());
		}
		cBox = new JComboBox(strings);
	    }
	}

    }

    /***************************************************/

    private class GroupingInfo extends JPanel implements ActionListener {

	private InfomatModel model;
	private IObjectGrouping grouping;
	private boolean row;
	private JTextField groupingNameTextField;

	public GroupingInfo(IObjectGrouping grouping, InfomatModel model, boolean row){

	    //super(new BorderLayout());
	    super(new GridLayout(2,1));
	    this.model=model;
	    this.grouping=grouping;
	    this.row=row;

	    putAll();
	}

	public void putAll(){
	    removeAll();
	    //String text = "<html><h3>"+rowOrColumn+" Grouping: <b>"+grouping.getName()+"</h3></b></html>"
	    String text = "<html><h2>"+grouping.getName()+"</h2></html>";
	    JLabel l = new JLabel(text);
	    l.setHorizontalAlignment(SwingConstants.CENTER);
	    add(l);


	    int height = (int) (l.getPreferredSize().getHeight());
	    int width = (int) (getPreferredSize().getWidth());
	    //int width = (int) (l.getPreferredSize().getWidth());
	    Dimension d = new Dimension(width, height);  		
	    setPreferredSize(d);
	    setMinimumSize(d);
	    setMaximumSize(d);

	    revalidate();
	    repaint();
	}


	public void actionPerformed(ActionEvent e) {
	    if(e.getSource() instanceof JTextField){
		JTextField t = (JTextField) e.getSource();
		IObjectGrouping grouping = 
		    model.getGUIIMatrix().getCurrentGrouping(row);
		String oldName = grouping.getName();
		if(t == groupingNameTextField){
		    String newName = t.getText();
		    Vector<IObjectGrouping> groupings = 
			model.getGUIIMatrix().getGroupings(row).getGroupings();
		    for(int i=0; i<groupings.size(); i++){
			if(newName.equals(groupings.elementAt(i).getName()))
			    newName=oldName;
		    }
		    GUIGroupingsForGroup guigfg = 
			model.getGUIIMatrix().getGroupings(row);
		    guigfg.removeGrouping(grouping);
		    grouping.setName(newName);
		    guigfg.setGrouping(grouping);

		    model.update();
		}
		putAll();
	    }
	}







    }

    /***************************************************/

    private class SortingChoicePanel extends JPanel{

	private String[] sortingMeasureOptions = {"Purity", "Entropy", "Part Purity", "Self Similarity"};
	private JComboBox cBox;

	public SortingChoicePanel(SorterPanel sorterPanel){

	    //			setBorder(new TitledBorder("Sorting options"));
	    setBorder(new LineBorder(Color.BLACK));
	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	    setAlignmentX(CENTER_ALIGNMENT);


	    JTextField dummyTextField = new JTextField("123");
	    int height = (int) dummyTextField.getPreferredSize().getHeight();

	    int pheight = (int) (this.getPreferredSize().getHeight());
	    int pwidth = (int) (panelDimension.getWidth());
	    //			int pwidth = 300;
	    Dimension pd = new Dimension(pwidth, height*2);
	    setPreferredSize(pd);
	    setMinimumSize(pd);
	    setMaximumSize(pd);
	    //			sort.setAlignmentX(CENTER_ALIGNMENT);


	    JLabel m = new JLabel("Choose base for sorting: ");
	    m.setAlignmentX(CENTER_ALIGNMENT);

	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	    cBox = new JComboBox(sortingMeasureOptions);


	    Dimension d = new Dimension((int)cBox.getPreferredSize().getWidth(), height);

	    cBox.setPreferredSize(d);
	    cBox.setMinimumSize(d);
	    cBox.setMaximumSize(d);

	    cBox.addActionListener(sorterPanel);
	    //			cBox.setSelectedIndex(0);

	    add(m);
	    add(cBox);
	}

	public String getSelected(){
	    return (String) cBox.getSelectedItem();
	}

	public JComboBox getComboBox(){
	    return this.cBox;
	}
    }





    /***************************************************/
    private class Ordering extends JPanel
	implements ActionListener{

	private InfomatModel model;
	private InfomatView view;
	private SorterPanel sorterPanel;

	private JButton button;
	private JButton incDecButton;

	public Ordering(InfomatModel model, InfomatView view, SorterPanel sorterPanel){
	    this.model=model;
	    this.view=view;
	    this.sorterPanel=sorterPanel;

	    incDecButton = new JButton("Descending");
	    incDecButton.addActionListener(this);
	    add(incDecButton);



	    button = new JButton("Sort by value");
	    button.addActionListener(this);
	    add(button);

	    JTextField dummyTextField = new JTextField("Descending");
	    int height = (int) dummyTextField.getPreferredSize().getHeight();
	    JButton dummy = new JButton("Descending");
	    int width = (int) dummy.getPreferredSize().getWidth();
	    button.setPreferredSize(new Dimension((int)button.getPreferredSize().getWidth(), height));
	    incDecButton.setPreferredSize(new Dimension(width, height));


	    Dimension d = getPreferredSize(); //new Dimension(width, height);
	    setPreferredSize(d);
	    setMinimumSize(d);
	    setMaximumSize(d);

	}

	public void actionPerformed(ActionEvent e) {
	    if(e.getSource() instanceof JButton){
		JButton b = (JButton) e.getSource();
		if(b==button){
		    boolean descending = true;
		    if(incDecButton.getText().equals("Ascending"))
			descending=false;
		    MeasureSorter.sortGroupingByGroupMeasureMap(sorterPanel.getCurrentGrouping(), sorterPanel.getGroupMeasureMap(), descending);
		    sorterPanel.getCurrentGrouping().sort();
		    //groups must also be "recreated" if a sorting is done.
		    IObjectGrouping currentGrouping = 
			sorterPanel.getCurrentGrouping();
		    groups = new Groups(currentGrouping, rowPanel, groupMeasureMap);
		    view.resetImage();
		    view.forceChange();
		    model.update();

		}
		else if(b==incDecButton){
		    if(incDecButton.getText().equals("Descending"))
			incDecButton.setText("Ascending");
		    else if(incDecButton.getText().equals("Ascending"))
			incDecButton.setText("Descending");
		    repaint();

		}


	    }

	}

    }

    /***************************************************/

    private class SorterMethodPanel extends JPanel{
	protected JButton calc;
	protected JLabel groupChoice;
	public SorterMethodPanel(SorterPanel sorterPanel){
	    //			setBorder(new TitledBorder("Method settings"));
	    setBorder(new LineBorder(Color.BLACK));
	    groupChoice = new JLabel("Choose reference group: ");
	    groupChoice.setAlignmentX(CENTER_ALIGNMENT);

	    JTextField dummyTextField = new JTextField("Calculate");
	    int height = (int) dummyTextField.getPreferredSize().getHeight();
	    JButton dummy = new JButton("Descending");

	    int pheight = (int) (this.getPreferredSize().getHeight());
	    //			int pwidth = (int) (super.getPreferredSize().getWidth());
	    int pwidth = 300;
	    Dimension pd = new Dimension(pwidth, height*3);
	    setPreferredSize(pd);
	    setMinimumSize(pd);
	    setMaximumSize(pd);

	    this.calc=new JButton("Calculate");

	    int width = (int) dummy.getPreferredSize().getWidth();
	    calc.setPreferredSize(new Dimension(width, height));

	    calc.setAlignmentX(CENTER_ALIGNMENT);

	    calc.addActionListener(sorterPanel);
	}
	public JButton getCalculationButton(){

	    return calc;
	}

    }

    private class ExtrinsicGroupMeasureSorterPanel extends SorterMethodPanel{

	private String measureName;
	private JComboBox cBox;
	private boolean row;

	public ExtrinsicGroupMeasureSorterPanel(SorterPanel sorterPanel, InfomatModel model, boolean row, String measureName){
	    super(sorterPanel);
	    this.measureName=measureName;
	    this.row=row;

	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	    Vector<IObjectGrouping> groupings = 
		model.getGUIIMatrix().getGroupings(row).getGroupings();
	    
	    if(groupings != null && groupings.size()>0){
		Vector<String> strings = new Vector<String>();

		for(int i=0; i<groupings.size(); i++)
		    strings.add(groupings.elementAt(i).getName());

		cBox = new JComboBox(strings);
		JTextField dummyTextField = new JTextField("123");
		int height = (int) dummyTextField.getPreferredSize().getHeight();

		Dimension d = new Dimension((int)cBox.getPreferredSize().getWidth(), height);

		cBox.setPreferredSize(d);
		cBox.setMinimumSize(d);
		cBox.setMaximumSize(d);


		//				cBox.setSelectedIndex(0);
		add(groupChoice);
		add(cBox);
		add(calc);
	    }
	}

	public IObjectGrouping getCurrentGrouping(){
	    String newSelected = (String) cBox.getSelectedItem();

	    return model.getGUIIMatrix().getGroupings(row).getGroupingWithName(newSelected);
	}

	public String getMeasureName(){
	    return measureName;
	}

	public void setMeasureName(String measureName){
	    this.measureName=measureName;
	}

	public JComboBox getComboBox(){
	    return this.cBox;
	}

    }


    /***************************************************/

    private class GroupsHeader extends JPanel{

	private SorterMethodPanel smp;

	public GroupsHeader(SorterMethodPanel smp){
	    this.smp=smp;
	    if(smp==null){
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		JLabel l = new JLabel("Measure: ");
		add(l);
		JLabel ref = new JLabel("Reference Grouping: ");
		add(ref);
		int height = (int) (l.getPreferredSize().getHeight());
		int width = (int) (getPreferredSize().getWidth());
		//int width = (int) (l.getPreferredSize().getWidth());
		Dimension d = new Dimension(width, 2*height);  		
		setPreferredSize(d);
		setMinimumSize(d);
		setMaximumSize(d);
		//				return;
	    }
	    //			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	    if(smp instanceof ExtrinsicGroupMeasureSorterPanel){
		setLayout(new GridLayout(2,1));

		ExtrinsicGroupMeasureSorterPanel egmsp = (ExtrinsicGroupMeasureSorterPanel) sorterMethodPanel;
		IObjectGrouping ioging = egmsp.getCurrentGrouping();
		JLabel l = new JLabel("Measure: "+egmsp.getMeasureName(), JLabel.LEFT);
		add(l);
		JLabel ref = new JLabel("Reference Grouping: " + ioging.getName(), JLabel.LEFT);
		add(ref);
		//				JLabel refName = new JLabel(ioging.getName());
		//				add(refName);



		int height = (int) (l.getPreferredSize().getHeight());
		int width = (int) (getPreferredSize().getWidth());
		//int width = (int) (l.getPreferredSize().getWidth());
		Dimension d = new Dimension(width, 2*height);  		
		setPreferredSize(d);
		setMinimumSize(d);
		setMaximumSize(d);

	    }
	}
    }

    /***************************************************/

    private class Groups extends JScrollPane{

	private IObjectGrouping grouping;
	private boolean rowPanel;
	private HashMap<IObjectGroup, Measure> groupMeasureMap;

	public Groups(IObjectGrouping grouping, boolean rowPanel, HashMap<IObjectGroup, Measure> groupMeasureMap){
	    this.grouping=grouping;
	    this.rowPanel=rowPanel;
	    JPanel p = new JPanel();

	    Vector<IObjectGroup> groups = grouping.getGroups();
	    p.setLayout(new GridLayout(groups.size(), 1));

	    int height=0;
	    int width=0;
	    if(groupMeasureMap!=null){
		for(int i=0; i<groups.size(); i++){
		    //		GroupInfo gi = new GroupInfo(this, i+1, groups.elementAt(i));
		    GroupInfo gi = null;
		    gi = 
			new GroupInfo(i+1, grouping, groups.elementAt(i), groupMeasureMap,
				      rowPanel);
		    height+=gi.getPreferredSize().getHeight();
		    width+=gi.getPreferredSize().getWidth();
		    p.add(gi);
		}
	    }

	    setPreferredSize(new Dimension(width, height));
	    setBorder(new LineBorder(Color.BLACK));
	    setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

	    /*
	      Dimension d = new Dimension(10, (int)getPreferredSize().getHeight());
	      getVerticalScrollBar().setPreferredSize(d);
	      getVerticalScrollBar().setMinimumSize(d);
	      getVerticalScrollBar().setMaximumSize(d);
	      getVerticalScrollBar().setSize(d);
	    */

	    setViewportView(p);
	}

	/*
	  public void paintComponent(Graphics g) {
	  super.paintComponent(g);
	  }
	*/
	public void setGroupMeasureMap(HashMap<IObjectGroup, Measure> groupMeasureMap){
	    this.groupMeasureMap=groupMeasureMap;
	}

    }

    /***************************************************/
    private class GroupInfo extends JPanel
	implements ActionListener{

	private IObjectGrouping grouping;
	private IObjectGroup group;
	private HashMap<IObjectGroup, Measure> groupMeasureMap;
	private int order;
	private boolean rowPanel;
	//	ShowGroupingSettings sgs;
	private JTextField textField;


	public GroupInfo(int order, IObjectGrouping grouping, IObjectGroup group, HashMap<IObjectGroup, Measure> groupMeasureMap,
			 boolean rowPanel){
	    this.grouping=grouping;
	    this.group=group;
	    this.groupMeasureMap=groupMeasureMap;
	    this.order=order;
	    this.rowPanel=rowPanel;

	    setLayout(new GridLayout(2,1));

	    if(groupMeasureMap.get(group) instanceof MultiMeasure){
		setLayout(new GridLayout(3,1));
	    }

	    putAll();
	    setBorder(new LineBorder(Color.BLACK));

	    /*
	      Dimension d= new 
	      Dimension((int) textField.getPreferredSize().getWidth(), 
	      ShowGroupingSettings.textHeight);
	      //Dimension d = textField.getPreferredSize();
	      setPreferredSize(d);
	      setMinimumSize(getPreferredSize());
	      setMaximumSize(getPreferredSize());
	    */
	}

	//		public void paintComponent(Graphics g) {
	//		super.paintComponent(g);

	//		}

	public void putAll(){
	    removeAll();
	    int height = (int) (new JTextField("Dummy")).getPreferredSize().getHeight();
	    // Name
	    String name = ""+order+". "+group.getName()+"  ("+group.numberOfIObjects();
	    //String name = "<html><h3>"+order+". "+group.getName()+"  ("+group.size()+")</h3></html>";
	    if(rowPanel)
		name += " rows)";
	    else
		name += " columns)";
	    JLabel label = new JLabel(name);
	    //label.setFont(new Font(Font.BOLD));
	    add(label);
	    //int height = (int) label.getPreferredSize().getHeight();
	    int nr = String.valueOf(order).length()+2;
	    String start = "";
	    for(int i=0; i<nr; i++)
		start+=" ";

	    // Change Name
	    JPanel part = new JPanel();

	    //part.setLayout(new GridLayout(1,2));
	    part.setLayout(new BoxLayout(part, BoxLayout.X_AXIS));

	    if(groupMeasureMap!=null){
		Measure measure = groupMeasureMap.get(group);
		if(measure==null)
		    label = new JLabel(start+"Value: ");
		else if(measure instanceof MultiMeasure){

		    //		    String ls = start+measure.getName()+" value: "+Round.roundDecimalsToTrimmedString(measure.getValue(), 2); 
		    String ls = start+measure.getName()+" value: "+measure.getValue(); 
		    if(measure.getRelativeIObjectGroup()!=null)
			ls += " ("+measure.getRelativeIObjectGroup().getName()+")";	

		    label = new JLabel(ls);
		    part.add(label);
		    add(part);

		    part = new JPanel();
		    part.setLayout(new BoxLayout(part, BoxLayout.X_AXIS));

		    ls = "";
		    MultiMeasure mm = (MultiMeasure) measure;
		    Vector<Measure> mmv = mm.getMeasures();
		    for(int i = 0; i < mmv.size(); i++){
			Measure m = mmv.get(i);
			ls += start+m.getName()+": "+Round.roundDecimalsToTrimmedString(m.getValue(), 2);
			if(m.getRelativeIObjectGroup()!=null)
			    ls += " ("+m.getRelativeIObjectGroup().getName()+")";
			ls += ", ";

		    }

		    label = new JLabel(ls);
		    part.add(label);
		    add(part);

		}
		else if(measure instanceof Measure){
		    //		    String ls = start+measure.getName()+" value: "+Round.roundDecimalsToTrimmedString(measure.getValue(), 2); 
		    String ls = start+measure.getName()+" value: "+measure.getValue(); 		    if(measure.getRelativeIObjectGroup()!=null)
			ls += " ("+measure.getRelativeIObjectGroup().getName()+")";	

		    label = new JLabel(ls);
		    part.add(label);
		    add(part);
		}

	    }


	    /*
	      height = (int) textField.getPreferredSize().getHeight();
	      int width = (int) textField.getPreferredSize().getWidth();
	      int heightMult=2;
	      if(colorPanel)
	      heightMult=3;
	      Dimension d = new Dimension(2*width, heightMult*height);
	    */
	    Dimension d = getPreferredSize();
	    setPreferredSize(d);
	    setMinimumSize(d);
	    setMaximumSize(d);

	    /*
	      d=getPreferredSize();
	      setMaximumSize(d);
	      setMinimumSize(d);
	    */

	    revalidate();
	    repaint();
	}

	public void actionPerformed(ActionEvent e) {
	    if(e.getSource() instanceof JTextField){    	    
		String text = textField.getText();
		//System.out.println("-------------Change: "+text);
		group.setName(text);
	    }else if(e.getSource() instanceof JButton){
		JButton jb = (JButton) e.getSource();
	    }
	    putAll();
	}


    }





}
