package infomat.view.tools;

import infomat.evaluation.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.view.*;
import infomat.view.grouping.settings.*;
import infomat.view.evaluation.*;
import infomat.view.util.*;

import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;
import mro.util.measures.*;
import mro.util.measures.dialog.*;


import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;
import javax.swing.border.*;

import java.util.*;
import java.io.*;

import javax.swing.*;


/**
 * For evaluation in the GUI.
 *
 * @version          080619
 * @author           Magnus Rosell
 *
 */
public class EvaluationFrame extends JFrame{
    public Dimension ALGORITHMFRAME_PREFERRED_SIZE = 
	new Dimension(300, 800);


    public EvaluationFrame(InfomatModel model){
	super("Evaluation");
	if(model.getIMatrix()!=null){
	    Container container = getContentPane();
	    container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));

	    EvaluationPanel evalPanel = new EvaluationPanel(model);
	    container.add(evalPanel);
	    setVisible(true);
	    setPreferredSize(ALGORITHMFRAME_PREFERRED_SIZE);
	    pack();
	}
	else{
	    JOptionPane
		.showMessageDialog(
				   this,
				   "Nothing to evaluate!",
				   "Evaluation",
				   JOptionPane.INFORMATION_MESSAGE);
	}
    }

    private class EvaluationPanel extends JPanel
	implements ActionListener{

	private InfomatModel model;

	private JPanel infoAndButtons;
	private JButton rowsOrColumns;
	private AlternatingButton fitGroupingsB;
	private JButton apply;

	private EvaluationSelectionPanel evaluationSelectionPanel1;
	private EvaluationSelectionPanel evaluationSelectionPanel2;

	private MeasuresPanel measuresPanel;	



	public EvaluationPanel(InfomatModel model){
	    super();
	    this.model=model;
	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

	    //			Default: rowIObjectSet
	    boolean row = true;
	    //	    IObjectSet i = model.getIMatrix().getRowIObjectSet();

	    JLabel evaluateLabel = new JLabel("Choose grouping to evaluate:");
	    evaluateLabel.setAlignmentX(LEFT_ALIGNMENT);

	    evaluationSelectionPanel1 = new EvaluationSelectionPanel(model, "Choose grouping to evaluate:");
	    evaluationSelectionPanel1.makeComboBox(row);
	    evaluationSelectionPanel1.setAlignmentX(LEFT_ALIGNMENT);

	    int width = (int) ALGORITHMFRAME_PREFERRED_SIZE.getWidth();
	    int height = (int) evaluationSelectionPanel1.getPreferredSize().getHeight();
	    /*
	    JTextField dummyTextField = new JTextField("123");
	    int height = (int) dummyTextField.getPreferredSize().getHeight();
	    */

	    Dimension d = new Dimension(width, height);
	    evaluationSelectionPanel1.setPreferredSize(d);
	    /*
	    evaluationSelectionPanel1.setMaximumSize(d);
	    evaluationSelectionPanel1.setMinimumSize(d);
	    */


	    JLabel referenceLabel = new JLabel("Choose reference grouping:");
	    referenceLabel.setAlignmentX(LEFT_ALIGNMENT);

	    evaluationSelectionPanel2 = new EvaluationSelectionPanel(model, "Choose reference grouping:");
	    evaluationSelectionPanel2.makeComboBox(row);
	    evaluationSelectionPanel2.setAlignmentX(LEFT_ALIGNMENT);

	    /*
	    height = (int) evaluationSelectionPanel2.getPreferredSize().getHeight();
	    d = new Dimension(width, height);
	    */
	    evaluationSelectionPanel2.setPreferredSize(d);
	    /*
	    evaluationSelectionPanel2.setMaximumSize(d);
	    evaluationSelectionPanel2.setMinimumSize(d);
	    */

	    String path = (String) model.getPropertyValue("Result Path");
	    measuresPanel = new mro.util.measures.dialog.MeasuresPanel(path);
	    measuresPanel.setFileable(true);
	    measuresPanel.setAlignmentX(LEFT_ALIGNMENT);

	    JTextField dummy = new JTextField("Apply");
	    JButton dummyButton = new JButton("Cancel");
	    height = (int) dummy.getPreferredSize().getHeight();
	    width = (int) dummyButton.getPreferredSize().getWidth();
	    Dimension buttonSize = new Dimension(width, height);

	    JPanel rc = new JPanel();
	    rc.setLayout(new BoxLayout(rc, BoxLayout.X_AXIS));
	    JLabel rcChoice = new JLabel("Choose rows or columns: ");
	    rowsOrColumns = new JButton("Rows");
	    rowsOrColumns.setPreferredSize(buttonSize);
	    rowsOrColumns.addActionListener(this);
	    rc.add(rcChoice);
	    rc.add(rowsOrColumns);
	    rc.setAlignmentX(LEFT_ALIGNMENT);


	    JPanel purgeP = new JPanel();
	    purgeP.setLayout(new BoxLayout(purgeP, BoxLayout.X_AXIS));
	    JLabel purgeL = new JLabel("Fit groupings to each other: ");
	    String[] alt = {"True", "False"};
	    fitGroupingsB = new AlternatingButton(alt);	    
	    fitGroupingsB.setPreferredSize(buttonSize);
	    fitGroupingsB.addActionListener(this);
	    purgeP.add(purgeL);
	    purgeP.add(fitGroupingsB);
	    purgeP.setAlignmentX(LEFT_ALIGNMENT);

	    infoAndButtons = new JPanel();
	    infoAndButtons.setLayout(new BoxLayout(infoAndButtons, BoxLayout.Y_AXIS));
	    JPanel groupingChoice = new JPanel();
	    groupingChoice.add(rc);
	    groupingChoice.add(purgeP);
	    groupingChoice.add(evaluateLabel);
	    groupingChoice.add(evaluationSelectionPanel1);
	    groupingChoice.add(referenceLabel);
	    groupingChoice.add(evaluationSelectionPanel2);
	    groupingChoice.setLayout(new BoxLayout(groupingChoice, BoxLayout.Y_AXIS));
	    apply = new JButton("Evaluate");
	    apply.setPreferredSize(buttonSize);
	    apply.addActionListener(this);
	    apply.setAlignmentX(LEFT_ALIGNMENT);
			
	    /*
	    groupingChoice.setPreferredSize(new Dimension((int)groupingChoice.getPreferredSize().getWidth(), (int)evaluationSelectionPanel1.getPreferredSize().getHeight()*6));
	    */

	    infoAndButtons.add(groupingChoice);
	    infoAndButtons.setBorder(new TitledBorder("Groupings"));

	    infoAndButtons.add(apply);

	    infoAndButtons.setAlignmentX(LEFT_ALIGNMENT);
	    infoAndButtons.setMaximumSize(infoAndButtons.getPreferredSize());
	    infoAndButtons.setMinimumSize(infoAndButtons.getPreferredSize());
			
	    //			infoAndButtons.setBorder(new TitledBorder("Evaluation"));


	    putAll();
	}

	private void putAll(){
	    removeAll();

	    add(infoAndButtons);


	    /*
	    String path = (String) model.getPropertyValue("Result Path");
	    //	    oldMeasuresPanel = new infomat.view.evaluation.MeasuresPanel(path);
	    JScrollPane sp = new JScrollPane();
	    sp.setViewportView(oldMeasuresPanel);
	    sp.setBorder(new TitledBorder("Measures"));
	    add(sp);
	    */
	    
	    add(measuresPanel);

	    revalidate();
	    repaint();
			
	}

	public void paintcomponent(Graphics g){
	    super.paintComponent(g);
	}

	
	private boolean row(){
	    return rowsOrColumns.getText().equals("Rows");
	}


	public void actionPerformed(ActionEvent e) {
	    IMatrix matrix = model.getIMatrix();
	    if(e.getSource() instanceof JButton){
		JButton source = (JButton) e.getSource();

		if(source == apply){
		    //		    System.out.println("Apply----------");

		    /*
		    boolean row=true;
		    if(rowsOrColumns.getText().equals("Columns"))
			row=false;
		    IObjectSet set = matrix.getColumnIObjectSet();
		    if(row)
			set = matrix.getRowIObjectSet();
		    evaluationSelectionPanel1.makeComboBox(set);
		    evaluationSelectionPanel2.makeComboBox(set);
		    */
	
		    long timeStamp = System.currentTimeMillis();
		    IObjectGrouping evaluated = (IObjectGrouping)evaluationSelectionPanel1.getCurrentGrouping();
		    IObjectGrouping reference = (IObjectGrouping)evaluationSelectionPanel2.getCurrentGrouping();

		    if(evaluated!=null){

			if(reference!=null){

			    String purgeS = fitGroupingsB.getText();
			    if(purgeS.equals("True")){
				reference = new IObjectGrouping(reference);
				reference.purgeGroupingToGroup(evaluated);
				evaluated = new IObjectGrouping(evaluated);
				evaluated.purgeGroupingToGroup(reference);
			    }

			    Evaluation eval = new Evaluation(matrix, evaluated, reference, model.getISimilarity(row()), ""+timeStamp);
			    Measures measures = new Measures("Measures");
			    try{
				measures = eval.calculateAllMeasures();
			    }
			    catch(MeasureNameConflictException mnce){
				System.out.println("Measure Name Conflict! (Evaluation Frame)");
			    }
			    String path = (String) model.getPropertyValue("Result Path");
			    measuresPanel = new MeasuresPanel(measures, path);
			    measuresPanel.setFileable(true);
			    measuresPanel.setAlignmentX(LEFT_ALIGNMENT);
			    /*
			    ExtrinsicEvaluation eval = new ExtrinsicEvaluation(evaluated, reference, ""+timeStamp);
			    Measures measures = new Measures("Measures");
			    try{
				measures = eval.calculateAllMeasures();
			    }
			    catch(MeasureNameConflictException mnce){
				System.out.println("Measure Name Conflict! (Evaluation Frame)");
			    }
			    String path = (String) model.getPropertyValue("Result Path");
			    measuresPanel = new MeasuresPanel(measures, path);
			    measuresPanel.setFileable(true);
			    measuresPanel.setAlignmentX(LEFT_ALIGNMENT);
			    */
			}else{
			    //			    IntrinsicEvaluation intrinsicEval = new IntrinsicEvaluation(evaluated, ""+timeStamp);
			    Evaluation eval = new Evaluation(matrix, evaluated, model.getISimilarity(row()), ""+timeStamp);
			    Measures measures = new Measures("Measures");
			    try{
				measures = eval.calculateAllMeasures();
			    }
			    catch(MeasureNameConflictException mnce){
				System.out.println("Measure Name Conflict! (Evaluation Frame)");
			    }
			    String path = (String) model.getPropertyValue("Result Path");
			    measuresPanel = new MeasuresPanel(measures, path);
			    measuresPanel.setFileable(true);
			    measuresPanel.setAlignmentX(LEFT_ALIGNMENT);
			}

			putAll();
		    }
				
		}
		else if(source==rowsOrColumns){
		    boolean row = true;
		    if(rowsOrColumns.getText().equals("Columns")){
			rowsOrColumns.setText("Rows");
			row=true;
		    }
		    else if(rowsOrColumns.getText().equals("Rows")){
			rowsOrColumns.setText("Columns");
			row=false;
		    }
		    evaluationSelectionPanel1.makeComboBox(row);
		    evaluationSelectionPanel2.makeComboBox(row);
		}else if(source==fitGroupingsB){
		    fitGroupingsB.press();
		    putAll();
		}
	    }
	}
    }
}




