package infomat.view.tools;

import infomat.algorithms.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.view.*;
import infomat.view.grouping.settings.*;
import infomat.view.picture.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;

import javax.swing.*;



public class MatrixSummaryFrame extends JFrame{

    public Dimension ALGORITHMFRAME_PREFERRED_SIZE = 
	new Dimension(300, 500);

    private InfomatModel model;
    private InfomatView view;


    private JButton rowsOrColumns;
    private JButton apply;
    private JButton cancel;

    public MatrixSummaryFrame(InfomatModel model, InfomatView view){
	super("Matrix Summary");
	this.model=model;
	this.view=view;

	if(view==null || model==null || model.getIMatrix() == null){
	    dispose();
	    return;
	}

	Container container = getContentPane();
	container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));


	JTextField dummy = new JTextField("Apply");
	JButton dummyButton = new JButton("Cancel");
	int height = (int) dummy.getPreferredSize().getHeight();
	int width = (int) dummyButton.getPreferredSize().getWidth();
	Dimension buttonSize = new Dimension(width, height);

	PartInfo pirow = new PartInfo(model, true);
	pirow.setBorder(new TitledBorder("Rows"));
	pirow.setPreferredSize(new Dimension((int)this.getPreferredSize().getWidth(),(int)(this.getPreferredSize().getHeight()/2)));
	container.add(pirow);

	PartInfo picol = new PartInfo(model, false);
	picol.setBorder(new TitledBorder("Columns"));
	picol.setPreferredSize(new Dimension((int)this.getPreferredSize().getWidth(),(int)(this.getPreferredSize().getHeight()/2)));
	container.add(picol);


	setVisible(true);
	setPreferredSize(ALGORITHMFRAME_PREFERRED_SIZE);
	pack();
    }


    /********************************************************************/

    private class PartInfo extends JScrollPane{
	private boolean row;
	private IObjectSet ios;
	private String rowString;
	private JPanel p;
	public PartInfo(InfomatModel model, boolean row){
	    this.row=row;

	    IMatrix matrix = model.getIMatrix();

	    p = new JPanel();

	    int psize = 0;
	    int size = 0;

	    JTextField dummy = new JTextField("Apply");
	    JButton dummyButton = new JButton("Cancel");
	    int height = (int) dummy.getPreferredSize().getHeight();
	    int width = (int) dummyButton.getPreferredSize().getWidth();
	    Dimension buttonSize = new Dimension(width, height);

	    JPanel tot = new JPanel();
	    JPanel col = new JPanel();

	    tot.setLayout(new GridLayout(2,1));
	    if(!row){
		rowString = "words";
		ios = matrix.getColumnIObjectSet();
		JLabel colsize = new JLabel("Total: "+ios.numberOfIObjects()+" "+rowString);
		tot.add(colsize);
		JLabel avgWordOccurInNumTexts = new JLabel("Average number of texts that each word occurs in: " + matrix.avgRowsInCols());
		tot.add(avgWordOccurInNumTexts);
		//				JLabel avgTextsInWords = new JLabel("Average number of texts in words: " + matrix.avgRowsInCols());
	    }
	    else{
		rowString = "documents";
		ios = matrix.getRowIObjectSet();
		JLabel colsize = new JLabel("Total: "+ios.numberOfIObjects()+" "+rowString);
		tot.add(colsize);
		JLabel avgWordsInTexts = new JLabel("Average number of words in texts: " + matrix.avgColsInRows());
		tot.add(avgWordsInTexts);
	    }
	    p.add(tot);

	    //	    Vector<IObjectGrouping> iogingvec = ios.getGroupings();
	    Vector<IObjectGrouping> iogingvec = 
		model.getGUIIMatrix().getGroupings(row).getGroupings();
	    if(iogingvec.size()==1){
		JPanel no = new JPanel();
		no.setLayout(new GridLayout(1,1));
		no.add(new JLabel("No groupings"));
		col.add(no);
		psize++;
	    }
	    else{
		for(int i = 1; i < iogingvec.size(); i++){
		    psize++;
		    JPanel iogingp = new JPanel();
		    iogingp.setBorder(new LineBorder(Color.BLACK));

		    IObjectGrouping ioging = iogingvec.get(i);
		    JLabel grouping = new JLabel("*** "+ioging.getName());
		    iogingp.add(grouping);
		    Vector<IObjectGroup> iogvec = ioging.getGroups();

		    for(int j = 0; j < iogvec.size(); j++){
			size++;
			JPanel iogp = new JPanel();
			iogp.setLayout(new GridLayout(1,1));
			iogp.setBorder(new LineBorder(Color.BLACK));
			IObjectGroup iog = iogvec.get(j);
			String iogname=iog.getName();
			int iogsize=iog.numberOfIObjects();
			JLabel iogsizel = new JLabel(iogname+": "+iogsize+" "+rowString+" ("+Math.round((float)iogsize/ios.numberOfIObjects()*100)+"%)");
			iogp.add(iogsizel);
			//						iogp.setLayout(new GridLayout(size,1));
			iogingp.add(iogp);
		    }
		    iogingp.setLayout(new GridLayout(iogvec.size()+1, 1));
		    col.add(iogingp);
		}
	    }
	    //			col.setLayout(new GridLayout(psize, 1));
	    col.setLayout(new BoxLayout(col, BoxLayout.Y_AXIS));
	    p.add(col);
	    p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
	    //			p.setLayout(new GridLayout(2, 1));
	    add(p);
	    setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
	    setViewportView(p);
	    repaint();
	}

    }

}
