package mro.util.experimentation;

import mro.util.*;
import mro.util.doublehashmap.*;

import java.io.*;
import java.util.*; 

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.SAXException;


/**
 * RelativeEntry. Info on relative comparisons to appear in ResultTable.
 *
 * @version          080507
 * @author           Magnus Rosell
 */
public class RelativeEntry implements Comparable{

    private int relKind=-1; // 0 - row, 1 - column
                            // 2 - column, row
                            // 3 - row, column
    private int number=-1;
    private int relNumber=-1;
    private int place=-1;

    public void setDimension(String dimension){
	if(dimension.equals("r"))
	    this.relKind=0;
	else if(dimension.equals("c"))
	    this.relKind=1;
	else if(dimension.equals("cr"))
	    this.relKind=2;
	else if(dimension.equals("rc"))
	    this.relKind=3;
    }

    public void setNumber(String number){
	try{
	    this.number= Integer.parseInt(number);
	}catch(Exception e3){
	    System.out.println("Unsupported PathTableEntry Value!");
	    System.exit(0);
	}
    }
    
    public void setRelativeNumber(String relNumber){
	try{
	    this.relNumber= Integer.parseInt(relNumber);
	}catch(Exception e3){
	    System.out.println("Unsupported PathTableEntry Value!");
	    System.exit(0);
	}
    }
    
    public void setPlace(String place){
	try{
	    this.place= Integer.parseInt(place);
	}catch(Exception e3){
	    System.out.println("Unsupported PathTableEntry Value!");
	    System.exit(0);
	}
    }
    
    /*
    public boolean isRelativeRow(){
	return (relKind==0);
    }
    */

    public int getRelativeKind(){
	return relKind;
    }
    
    public int getNumber(){
	return number;
    }

    public int getRelativeNumber(){
	return relNumber;
    }

    public int getPlace(){
	return place;
    }


    public void printPartXML(PrintWriter pw) throws IOException{
	pw.print("<rele>");

	pw.print("<dim>");	
	if(relKind==0)
	    pw.print("r");
	else if(relKind==1)
	    pw.print("c");
	else if(relKind==2)
	    pw.print("cr");
	else if(relKind==3)
	    pw.print("rc");
	pw.print("</dim>");

	pw.print("<nr>");
	pw.print(getNumber());
	pw.print("</nr>");

	pw.print("<relnr>");
	pw.print(getRelativeNumber());
	pw.print("</relnr>");

	pw.println("</rele>");
    }

    public int compareTo(Object o){
	RelativeEntry re = (RelativeEntry) o;
	if(number<re.number)
	    return -1;
	else if(number>re.number)
	    return 1;
	return 0;
    }

}
