package mro.util.experimentation;

import java.io.*;
import java.util.Vector;
import java.util.HashMap;
import java.util.Collections;

import mro.util.*;
import mro.util.doublehashmap.*;
import mro.util.measures.*;

/**
 * Result Table.
 *
 * @version          080512
 * @author           Magnus Rosell
 */
public class ResultTable{

    // Implementera invertering ...

    private ResultTableStructure rts;
    private ResultPool resultPool;

    GenericOrderedDoubleHashMap<Integer, Integer, String> columnTitleMap=null;
    GenericOrderedDoubleHashMap<Integer, Integer, String> rowTitleMap=null;


    private GenericOrderedDoubleHashMap<Integer, Integer, Result> resultMap 
	= new GenericOrderedDoubleHashMap<Integer, Integer, Result>();


    public ResultTable(ResultTableStructure rts, ResultPool resultPool) throws MeasureNameConflictException{
	this.rts=rts;
	this.resultPool=resultPool;
	construct();
    }

    private void construct() throws MeasureNameConflictException{

	Table<String> columnTitleTable = rts.getColumnTitles();
	if(columnTitleTable != null)
	    columnTitleMap = columnTitleTable.getGenericOrderedDoubleHashMap();

	Table<String> rowTitleTable = rts.getRowTitles();
	if(rowTitleTable != null)
	    rowTitleMap = rowTitleTable.getGenericOrderedDoubleHashMap();

	PathTable pathTable = rts.getPathTable();
	String commonPath = pathTable.getCommonPath();
	GenericOrderedDoubleHashMap<Integer, Integer, PathEntry> 
	    pathMap = pathTable.getGenericOrderedDoubleHashMap();

	Vector<Integer> first = pathMap.getFirstIndexOrderVector();
	Vector<Integer> second = pathMap.getSecondIndexOrderVector();

	// Results
	for(int f=0; f<first.size(); f++){
	    for(int s=0; s<second.size(); s++){
		//		Vector<String> paths = 
		//		    pathMap.get(first.elementAt(f), second.elementAt(s));
		PathEntry pe = 
		    pathMap.get(first.elementAt(f), second.elementAt(s));

		Vector<String> paths = pe.getPaths();
		Result result = null;
		printStringVector(paths);
		for(int i=0; i<paths.size(); i++){
		    String path = "";
		    if(!commonPath.equals(""))
			path += commonPath + File.separator;
		    path += paths.elementAt(i);
		    
		    Result res = 
			new Result(resultPool.getResultForPath(path),
				   pe.getMeasureAbbreviation(),
				   pe.getRoundString());
		    //			System.out.println("res: "+res);
		    if(i==0){
			result = res;
		    }else{
			result.add(res);
		    }
		}

		resultMap.put(first.elementAt(f), second.elementAt(s), result);
	    }
	}
	RelativeTable relativeTable = rts.getRelativeTable();
	if(relativeTable!=null)
	    constructRelative(relativeTable);
    }

    
    private static <T> Vector<Vector<T>> getVectorVector(boolean first, GenericOrderedDoubleHashMap<Integer, Integer, T> map){
	Vector<Integer> firstV = map.getFirstIndexOrderVector();
	Vector<Integer> secondV = map.getSecondIndexOrderVector();


	Vector<Vector<T>> ret = new Vector<Vector<T>>();
	if(first){
	    for(int f=0; f<firstV.size(); f++){
		Vector<T> fv = new Vector<T>();
		for(int s=0; s<secondV.size(); s++){
		    fv.add(map.get(firstV.elementAt(f), secondV.elementAt(s)));
		}
		ret.add(fv);
	    }
	}else{
	    for(int s=0; s<secondV.size(); s++){
		Vector<T> sv = new Vector<T>();
		for(int f=0; f<firstV.size(); f++){
		    sv.add(map.get(firstV.elementAt(f), secondV.elementAt(s)));
		}
		ret.add(sv);
	    }
	}
	return ret;
    }


    private static <T> GenericOrderedDoubleHashMap<Integer, Integer, T>
	getGenericOrderedDoubleHashMap(boolean first, 
				       Vector<Vector<T>> vecVec){

	GenericOrderedDoubleHashMap<Integer, Integer, T> ret
	    = new GenericOrderedDoubleHashMap<Integer, Integer, T>();
	if(first){
	    for(int f=0; f<vecVec.size(); f++){
		Vector<T> vec = vecVec.elementAt(f);
		for(int s=0; s<vec.size(); s++){
		    ret.put(f, s, vec.elementAt(s));
		}
	    }
	}else{
	    for(int s=0; s<vecVec.size(); s++){
		Vector<T> vec = vecVec.elementAt(s);
		for(int f=0; f<vec.size(); f++){
		    ret.put(f, s, vec.elementAt(f));
		}
	    }
	}
	return ret;
    }



    private void insertRelativeInVecVec(Vector<Vector<Result>> resVecVec,
					Vector<RelativeEntry> relEntryVec){
	Collections.sort(relEntryVec);

	HashMap<RelativeEntry, Vector<Result>> entryRelResVecMap = 
	    new HashMap<RelativeEntry, Vector<Result>>();

	for(int i=0; i<relEntryVec.size(); i++){
	    RelativeEntry re = relEntryVec.elementAt(i);
	    int number = re.getNumber();
	    int relNumber = re.getRelativeNumber();
	    Vector<Result> numberVec = resVecVec.elementAt(number);
	    Vector<Result> relNumberVec = resVecVec.elementAt(relNumber);
	    Vector<Result> relResVec = new Vector<Result>();
	    for(int j=0; j<numberVec.size(); j++){
		Result numberResult = numberVec.elementAt(j);
		Result relNumberResult = relNumberVec.elementAt(j);
		String roundString = "2d"; // Two decimals
		Result relativeResult = 
		    numberResult.relative(relNumberResult, roundString);
		
		long time = System.currentTimeMillis();

		relResVec.add(relativeResult);
	    }
	    entryRelResVecMap.put(re, relResVec);
	}

	for(int shift=0; shift<relEntryVec.size(); shift++){
	    RelativeEntry re = relEntryVec.elementAt(shift);
	    int number = re.getNumber() + shift + 1;
	    Vector<Result> relResVec = entryRelResVecMap.get(re);
	    resVecVec.insertElementAt(relResVec, number);
	}	
    }

    private static <T> Vector<Vector<T>> 
	copyVectorVector(Vector<Vector<T>> in){

	Vector<Vector<T>> ret = new Vector<Vector<T>>();
	for(int i=0; i<in.size(); i++){
	    Vector<T> newVec = new Vector<T>(in.elementAt(i));
	    ret.add(newVec);
	}
	return ret;
    }
    

    private Vector<Vector<String>> 
	insertRelativeTitlesInVecVec(Vector<Vector<String>> titleVecVec, 
				     Vector<RelativeEntry> relEntryVec,
				     int diffSize){

	Vector<Vector<String>> newTitleVecVec = copyVectorVector(titleVecVec);



	for(int shift=0; shift<relEntryVec.size(); shift++){
	    RelativeEntry re = relEntryVec.elementAt(shift);
	    int number = re.getNumber() + diffSize;
	    Vector<String> numberTitleVec = 
		new Vector<String>(titleVecVec.elementAt(number));

	    int relNumber = re.getRelativeNumber() + diffSize;
	    Vector<String> relNumberTitleVec = 
		titleVecVec.elementAt(relNumber);

	    int relKind = re.getRelativeKind();
	    if(relKind == 0 || relKind == 1)
		numberTitleVec.set(numberTitleVec.size()-1, numberTitleVec.elementAt(numberTitleVec.size()-1)+" rel "+relNumberTitleVec.elementAt(numberTitleVec.size()-1));
	    
	    // Self relative
	    else if(relKind == 2 || relKind == 3)
		numberTitleVec.set(numberTitleVec.size()-1, "rel ("+numberTitleVec.elementAt(numberTitleVec.size()-1)+")");

	    newTitleVecVec.insertElementAt(numberTitleVec, number+shift+1);
	}	
	
	return newTitleVecVec;

	/*
	for(int shift=0; shift<relEntryVec.size(); shift++){
	    RelativeEntry re = relEntryVec.elementAt(shift);
	    int number = re.getNumber() + diffSize + shift;
	    Vector<String> numberTitleVec = 
		new Vector<String>(titleVecVec.elementAt(number));

	    int relNumber = re.getRelativeNumber() + diffSize + shift;
	    Vector<String> relNumberTitleVec = 
		titleVecVec.elementAt(relNumber);

	    int relKind = re.getRelativeKind();
	    if(relKind == 0 || relKind == 1)
		numberTitleVec.set(numberTitleVec.size()-1, numberTitleVec.elementAt(numberTitleVec.size()-1)+" rel "+relNumberTitleVec.elementAt(numberTitleVec.size()-1));
	    
	    // Self relative
	    else if(relKind == 2 || relKind == 3)
		numberTitleVec.set(numberTitleVec.size()-1, "rel ("+numberTitleVec.elementAt(numberTitleVec.size()-1)+")");

	    titleVecVec.insertElementAt(numberTitleVec, number+1);
	}	
	*/
    }

    private void constructRelative(RelativeTable relativeTable){
	System.out.println("constructRelative()");

	// Rows titles
	boolean first = true;
	Vector<Vector<String>> rowTitleVecVec = null;
	if(rowTitleMap!=null)
	    rowTitleVecVec = getVectorVector(first, rowTitleMap);

	// Column titles
	first = false;
	Vector<Vector<String>> columnTitleVecVec = null;
	if(columnTitleMap!=null)
	    columnTitleVecVec = getVectorVector(first, columnTitleMap);


	////// Rows
	first = true;
	Vector<Vector<Result>> resVecVec = getVectorVector(first, resultMap);
	Vector<RelativeEntry> relEntryVec = 
	    relativeTable.getRowRelativeVector();
	if(relEntryVec!=null){
	    insertRelativeInVecVec(resVecVec, relEntryVec);
	    resultMap = getGenericOrderedDoubleHashMap(first, resVecVec);

	    // Fixa rowtitles
	    int diffSize = 0;
	    /*
	    if(columnTitleVecVec!=null && rowTitleVecVec!=null){
		Vector<String> tmpVec = columnTitleVecVec.elementAt(0);
		diffSize = rowTitleVecVec.size() - tmpVec.size();
	    }
	    */
	    rowTitleVecVec = insertRelativeTitlesInVecVec(rowTitleVecVec, relEntryVec, diffSize);
	    rowTitleMap = getGenericOrderedDoubleHashMap(first, rowTitleVecVec);
	}

	////// Columns
	first = false;
	resVecVec = getVectorVector(first, resultMap);
	relEntryVec = 
	    relativeTable.getColumnRelativeVector();
	if(relEntryVec!=null){
	    //	    System.out.println("columns: "+resVecVec.size());

	    //	    relEntryVec = relativeTable.getColumnRelativeVector();
	    insertRelativeInVecVec(resVecVec, relEntryVec);
	    resultMap = getGenericOrderedDoubleHashMap(first, resVecVec);
	    
	    //	    System.out.println("columns: "+resVecVec.size());
	    
	    // Fixa columntitles
	    int diffSize = 0;
	    if(rowTitleVecVec!=null && columnTitleVecVec!=null){
		Vector<String> tmpVec = rowTitleVecVec.elementAt(0);
		diffSize = tmpVec.size();
	    }
	    columnTitleVecVec=insertRelativeTitlesInVecVec(columnTitleVecVec, relEntryVec, diffSize);
	    columnTitleMap = getGenericOrderedDoubleHashMap(first, columnTitleVecVec);
	}


	////// Columns of row relative
	first = false;
	resVecVec = getVectorVector(first, resultMap);
	relEntryVec = 
	    relativeTable.getColumnRowRelativeVector();
	if(relEntryVec!=null){
	    //	    System.out.println("columns: "+resVecVec.size());

	    insertSelfRelativeInVecVec(resVecVec, relEntryVec);
	    resultMap = getGenericOrderedDoubleHashMap(first, resVecVec);
	    
	    //	    System.out.println("columns: "+resVecVec.size());
	    
	    // Fixa columntitles
	    int diffSize = 0;
	    if(rowTitleVecVec!=null && columnTitleVecVec!=null){
		Vector<String> tmpVec = rowTitleVecVec.elementAt(0);
		diffSize = tmpVec.size();
	    }
	    columnTitleVecVec=insertRelativeTitlesInVecVec(columnTitleVecVec, relEntryVec, diffSize);
	    columnTitleMap = getGenericOrderedDoubleHashMap(first, columnTitleVecVec);
	}


	////// Rows of column relative
	first = true;
	resVecVec = getVectorVector(first, resultMap);
	relEntryVec = 
	    relativeTable.getRowColumnRelativeVector();
	if(relEntryVec!=null){
	    //	    System.out.println("columns: "+resVecVec.size());

	    insertSelfRelativeInVecVec(resVecVec, relEntryVec);
	    resultMap = getGenericOrderedDoubleHashMap(first, resVecVec);
	    
	    //	    System.out.println("columns: "+resVecVec.size());
	    
	    int diffSize=0;
	    rowTitleVecVec=insertRelativeTitlesInVecVec(rowTitleVecVec, relEntryVec, diffSize);
	    rowTitleMap = getGenericOrderedDoubleHashMap(first, rowTitleVecVec);
	}



    }


    private void insertSelfRelativeInVecVec(Vector<Vector<Result>> resVecVec,
					    Vector<RelativeEntry> relEntryVec){
	Collections.sort(relEntryVec);

	HashMap<RelativeEntry, Vector<Result>> entryRelResVecMap = 
	    new HashMap<RelativeEntry, Vector<Result>>();

	for(int i=0; i<relEntryVec.size(); i++){
	    RelativeEntry re = relEntryVec.elementAt(i);
	    int number = re.getNumber();
	    int relNumber = re.getRelativeNumber();
	    Vector<Result> numberVec = resVecVec.elementAt(number);
	    Result relRes = numberVec.elementAt(relNumber);
	    Vector<Result> relResVec = new Vector<Result>();
	    for(int j=0; j<numberVec.size(); j++){
		Result numberResult = numberVec.elementAt(j);
		String roundString = "2d";
		Result relativeResult = 
		    numberResult.relative(relRes, roundString);
		relResVec.add(relativeResult);
	    }
	    entryRelResVecMap.put(re, relResVec);
	}

	for(int shift=0; shift<relEntryVec.size(); shift++){
	    RelativeEntry re = relEntryVec.elementAt(shift);
	    int number = re.getNumber() + shift + 1;
	    Vector<Result> relResVec = entryRelResVecMap.get(re);
	    resVecVec.insertElementAt(relResVec, number);
	}	
    }










    /**
     * No error handling!
     */    
    private GenericOrderedDoubleHashMap<Integer, Integer, Measure>
	constructMeasureMap(){

	MeasureAbbreviationTable measureAbbreviationTable
	    = rts.getMeasureAbbreviationTable();
	PathTable pathTable = rts.getPathTable();
	GenericOrderedDoubleHashMap<Integer, Integer, PathEntry> 
	    pathMap = pathTable.getGenericOrderedDoubleHashMap();

	Vector<Integer> firstTmp = pathMap.getFirstIndexOrderVector();
	Vector<Integer> secondTmp = pathMap.getSecondIndexOrderVector();
	System.out.println("Path Table: "+pathTable+", ("+firstTmp.size()+", "+secondTmp.size()+")");


	GenericOrderedDoubleHashMap<Integer, Integer, Measure> map = 
	    new GenericOrderedDoubleHashMap<Integer, Integer, Measure>();
	
	Vector<Integer> first = resultMap.getFirstIndexOrderVector();
	Vector<Integer> second = resultMap.getSecondIndexOrderVector();

	System.out.println("Result Map: ("+first.size()+", "+second.size()+")");



	for(int f=0; f<first.size(); f++){
	    for(int s=0; s<second.size(); s++){
		Result result = 
		    resultMap.get(first.elementAt(f), second.elementAt(s));

		//		System.out.println(result);

		Measures measures = result.measures;

		/*
		System.out.println(f+"."+s);
		measures.save("/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/ExperimentatorTestJtteliten/avg/"+f+"."+s+".xml");
		*/

		/*
		PathEntry pe = 
		    pathMap.get(first.elementAt(f), second.elementAt(s));
		Vector<String> measureNames2 = 
		    measureAbbreviationTable.get(pe.getMeasureAbbreviation());
		*/
		Vector<String> measureNames = 
		    measureAbbreviationTable.get(result.measureAbbreviation);

		String name = measureNames.elementAt(0);
		if(measureNames.size()==1){
		    Measure m = measures.getMeasure(name);
		    map.put(first.elementAt(f), second.elementAt(s), m);
		}else{
		    //		    System.out.println("measures: "+measures);
		    //		    System.out.println("name: "+name);
		    MeasureGroup mg = measures.getMeasureGroup(name);
		    //		    System.out.println("mg: "+mg);
		    for(int i=1; i<measureNames.size()-1; i++){
			name = measureNames.elementAt(i);
			//			System.out.println("name: "+name);
			mg = mg.getMeasureGroup(name);
			//			System.out.println("mg: "+mg);
		    }
		    name = measureNames.elementAt(measureNames.size()-1);
		    Measure m = mg.getMeasure(name);
		    map.put(first.elementAt(f), second.elementAt(s), m);
		}
	    }
	}
	return map;
    }

    
    public void constructLatexTableFile(String fileName) throws IOException{
	File file = new File(fileName);
	constructLatexTableFile(file);
    }

    public void constructLatexTableFile(File file) throws IOException{
	System.out.println("constructLatexTableFile()");

	PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(file)));

	/*
	PathTable pathTable = rts.getPathTable();
	GenericOrderedDoubleHashMap<Integer, Integer, PathEntry> 
	    pathMap = pathTable.getGenericOrderedDoubleHashMap();
	*/

	
	LatexHelper.printLatexHead(pw);

	int textSize=1; // small


	if(columnTitleMap!=null){

	Vector<Integer> columnTitleMapFirst = 
	    columnTitleMap.getFirstIndexOrderVector();
	Vector<Integer> columnTitleMapSecond = 
	    columnTitleMap.getSecondIndexOrderVector();
	int numberOfColumns = columnTitleMapSecond.size();

	LatexHelper.printTableHead(pw, numberOfColumns, textSize);

	// Column titles
	System.out.println("Column titles: ");
	for(int f=0; f<columnTitleMapFirst.size(); f++){
	    Vector<String> columnStrings = new Vector<String>();

	    Vector<String> multis = new Vector<String>();
	    for(int s=0; s<columnTitleMapSecond.size(); s++){
		String columnTitleString = columnTitleMap.get(f, s);
		//		System.out.println("columnTitleString: "+columnTitleString);
		boolean multi = false;
		int t = s+1;
		while(t<columnTitleMapSecond.size()){
		    String nextString = columnTitleMap.get(f, t);
		    //		    System.out.println("nextString: "+nextString);
		    if(nextString.equals(columnTitleString)){
			t++;
		    }else{
			if(t>s+1){
			    columnTitleString = "\\multicolumn{"+(t-s)+
				"}{c|}{"+columnTitleString+"}";
			    multis.add(""+s+"-"+t);
			    s += t-s-1;
			    //			    multis.add(""+(s-1)+"-"+t);
			    multi=true;
			    //			    System.out.println("->: "+columnTitleString);
			}
			break;
		    }
		    if(t>=columnTitleMapSecond.size()){
			columnTitleString = "\\multicolumn{"+(t-s)+
			    "}{c|}{"+columnTitleString+"}";
			multis.add(""+s+"-"+t);
			s += t-s-1;
			//			multis.add(""+(s-1)+"-"+t);
			multi=true;
			//			System.out.println("->: "+columnTitleString);
			break;
		    }
		    
		}
		if(!multi){
		    columnTitleString = "\\multicolumn{"+1+
			"}{c|}{"+columnTitleString+"}";
		}


		columnStrings.add(columnTitleString);
		System.out.print(columnTitleString+", ");
	    }
	    System.out.println();
	    LatexHelper.printTableRow(pw, columnStrings);
	    
	    for(int m=0; m<multis.size(); m++){
		String multiString = multis.elementAt(m);
		LatexHelper.printTablePartHorizontalLine(pw, multiString);
	    }

	}

	}else{

	    int numberOfColumns = resultMap.getFirstIndexOrderVector().size();
	    LatexHelper.printTableHead(pw, numberOfColumns, textSize);

	}


	LatexHelper.printHLine(pw);
	

	Vector<Integer> rowTitleMapSecond = null;
	if(rowTitleMap!=null){
	    System.out.println("Rows: ");
	    // Row titles
	    rowTitleMapSecond = 
		rowTitleMap.getSecondIndexOrderVector();
	}

	// Values	
	GenericOrderedDoubleHashMap<Integer, Integer, Measure> measureMap = 
	    constructMeasureMap();
	Vector<Integer> first = measureMap.getFirstIndexOrderVector();
	Vector<Integer> second = measureMap.getSecondIndexOrderVector();
	for(int f=0; f<first.size(); f++){
	    Vector<String> rowStrings = new Vector<String>();

	    // Row titles
	    if(rowTitleMap!=null){
		for(int rs=0; rs<rowTitleMapSecond.size(); rs++){
		    String rowTitleString = rowTitleMap.get(f, rs);
		    rowStrings.add(rowTitleString);
		}
	    }

	    // Values
	    for(int s=0; s<second.size(); s++){
		Measure m = 
		    measureMap.get(first.elementAt(f), second.elementAt(s));

		/*
		PathEntry pe = pathMap.get(first.elementAt(f), second.elementAt(s));
		String roundString = pe.getRoundString();
		*/

		Result result = resultMap.get(first.elementAt(f), second.elementAt(s));
		String roundString = result.roundString;


		if(m!=null){
		    if(m instanceof AverageMeasure){
			AverageMeasure am = (AverageMeasure) m;
			//			String val = Round.roundDecimalsToTrimmedString(am.getValue(), 2);
			//			String std = Round.roundDecimalsToTrimmedString(am.getSTD(), 2);
			String val = Round.roundToTrimmedString(am.getValue(), roundString);
			if(am.getNumberOfMeasures()>1){

			    String std = Round.roundToTrimmedString(am.getSTD(), roundString);
			    rowStrings.add("$"+val+" \\  ("+std+")$"); 
			}else{
			    rowStrings.add("$"+val+"$");
			}
		    }else{
			//			String val = Round.roundDecimalsToTrimmedString(m.getValue(), 2);
			String val = Round.roundToTrimmedString(m.getValue(), roundString);
			rowStrings.add("$"+val+"$"); 
		    }
		}else{
		    System.out.println("Measure == null ");
		}
	    }
	    printStringVector(rowStrings);
	    LatexHelper.printTableRow(pw, rowStrings);
	}

	String caption="Table";
	String label="Table";
	if(rts.getCaption()!=null){
	    caption = rts.getCaption();
	    label = rts.getCaption();
	}
	LatexHelper.printTableEnd(pw, caption, label, textSize);
	//	LatexHelper.printTableEnd(pw);
	LatexHelper.printLatexEnd(pw);
	pw.close();

	System.out.println("constructLatexTableFile: "+file.getPath());

    }

    private void printStringVector(Vector<String> vec){
	for(int i=0; i<vec.size(); i++){
	    System.out.print(vec.elementAt(i)+", ");
	}
	System.out.println();
    }


    public void constructMatlabMatrixFile(String fileName) throws IOException{
	File file = new File(fileName);
	constructMatlabMatrixFile(file);
    }

    public void constructMatlabMatrixFile(File file) throws IOException{
	System.out.println("constructMatlabMatrixFile()");
	
	int kind = 0; // Matlab
	constructMatrixFile(file, kind);
	System.out.println("constructMatrixFile: "+file.getPath());
    }


    public void constructCSVMatrixFile(String fileName) throws IOException{
	File file = new File(fileName);
	constructCSVMatrixFile(file);
    }

    public void constructCSVMatrixFile(File file) throws IOException{
	System.out.println("constructCSVMatrixFile()");
	
	int kind = 1; // csv
	constructMatrixFile(file, kind);
	System.out.println("constructMatrixFile: "+file.getPath());
    }


    /**
     * "kind" -> 0 - Matlab matrix
     *           1 - csv (MS Excel)
     */
    public void constructMatrixFile(File file, int kind) throws IOException{

	PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(file)));

	Vector<Vector<String>> vals = new Vector<Vector<String>>();
	Vector<Vector<String>> stds = new Vector<Vector<String>>();

	// Values	
	GenericOrderedDoubleHashMap<Integer, Integer, Measure> measureMap = 
	    constructMeasureMap();
	Vector<Integer> first = measureMap.getFirstIndexOrderVector();
	Vector<Integer> second = measureMap.getSecondIndexOrderVector();
	for(int f=0; f<first.size(); f++){
	    Vector<String> rowVals = new Vector<String>();
	    Vector<String> rowStds = new Vector<String>();

	    // Values
	    for(int s=0; s<second.size(); s++){
		Measure m = 
		    measureMap.get(first.elementAt(f), second.elementAt(s));

		Result result = resultMap.get(first.elementAt(f), second.elementAt(s));
		String roundString = result.roundString;

		if(m!=null){
		    if(m instanceof AverageMeasure){
			AverageMeasure am = (AverageMeasure) m;

			String val = Round.roundToTrimmedString(am.getValue(), roundString);
			String std = Round.roundToTrimmedString(am.getSTD(), roundString);

			rowVals.add(""+val);
			rowStds.add(""+std);
		    }else{
			//			String val = Round.roundDecimalsToTrimmedString(m.getValue(), 2);
			String val = Round.roundToTrimmedString(m.getValue(), roundString);
			rowVals.add(""+val);
		    }
		}else{
		    System.out.println("Measure == null ");
		}
	    }
	    vals.add(rowVals);
	    stds.add(rowStds);

	}

	
	if(kind == 0){ // Matlab

	    MatlabHelper.printMatrixHead(pw, "M");
	    for(int i=0; i<vals.size(); i++){
		Vector<String> rowVals = vals.elementAt(i);
		MatlabHelper.printMatrixRow(pw, rowVals);
	    }
	    MatlabHelper.printMatrixEnd(pw);
	    
	    MatlabHelper.printMatrixHead(pw, "stdM");
	    for(int i=0; i<stds.size(); i++){
		Vector<String> rowVals = stds.elementAt(i);
		MatlabHelper.printMatrixRow(pw, rowVals);
	    }
	    MatlabHelper.printMatrixEnd(pw);
	    
	} else if(kind == 1){ // CSV - MS Excel
	    
	    pw.println("Matrix");
	    for(int i=0; i<vals.size(); i++){
		Vector<String> rowVals = vals.elementAt(i);
		for(int j=0; j<rowVals.size(); j++)
		    pw.print(rowVals.elementAt(j)+"; ");
		pw.println();
	    }
	    pw.println();
	    
	    pw.println("Standard Deviation");
	    for(int i=0; i<stds.size(); i++){
		Vector<String> rowVals = stds.elementAt(i);
		for(int j=0; j<rowVals.size(); j++)
		    pw.print(rowVals.elementAt(j)+"; ");
		pw.println();
	    }
	    pw.println();

	}

	pw.close();
    }







    /*
    public void constructMatlabMatrixFile(File file) throws IOException{
	System.out.println("constructMatlabMatrixFile()");

	PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(file)));

	Vector<Vector<String>> vals = new Vector<Vector<String>>();
	Vector<Vector<String>> stds = new Vector<Vector<String>>();

	// Values	
	GenericOrderedDoubleHashMap<Integer, Integer, Measure> measureMap = 
	    constructMeasureMap();
	Vector<Integer> first = measureMap.getFirstIndexOrderVector();
	Vector<Integer> second = measureMap.getSecondIndexOrderVector();
	for(int f=0; f<first.size(); f++){
	    Vector<String> rowVals = new Vector<String>();
	    Vector<String> rowStds = new Vector<String>();

	    // Values
	    for(int s=0; s<second.size(); s++){
		Measure m = 
		    measureMap.get(first.elementAt(f), second.elementAt(s));

		Result result = resultMap.get(first.elementAt(f), second.elementAt(s));
		String roundString = result.roundString;

		if(m!=null){
		    if(m instanceof AverageMeasure){
			AverageMeasure am = (AverageMeasure) m;

			String val = Round.roundToTrimmedString(am.getValue(), roundString);
			String std = Round.roundToTrimmedString(am.getSTD(), roundString);

			rowVals.add(""+val);
			rowStds.add(""+std);
		    }else{
			//			String val = Round.roundDecimalsToTrimmedString(m.getValue(), 2);
			String val = Round.roundToTrimmedString(m.getValue(), roundString);
			rowVals.add(""+val);
		    }
		}else{
		    System.out.println("Measure == null ");
		}
	    }
	    vals.add(rowVals);
	    stds.add(rowStds);

	}


	MatlabHelper.printMatrixHead(pw, "M");
	for(int i=0; i<vals.size(); i++){
	    Vector<String> rowVals = vals.elementAt(i);
	    MatlabHelper.printMatrixRow(pw, rowVals);
	}
	MatlabHelper.printMatrixEnd(pw);

	MatlabHelper.printMatrixHead(pw, "stdM");
	for(int i=0; i<stds.size(); i++){
	    Vector<String> rowVals = stds.elementAt(i);
	    MatlabHelper.printMatrixRow(pw, rowVals);
	}
	MatlabHelper.printMatrixEnd(pw);


	pw.close();

	System.out.println("constructMatrixFile: "+file.getPath());
    }

    */






}

